// DRGTexture.h
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
// DRGTexture.h: declaration of the CDRGTexture class.
// Originally adapted from some DX6 sample code. Added some robustness and extra 
// functionality (alpha bitmaps, creating inverse hieghtmaps for embossing, automatic
// resize for HW requirements, etc)
//
// ------------------------------------------------------------------------------------
//
// Authors: Dean Macri, Kim Pallister - Intel Developer Relations & Engineering Group
//
////////////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_DRGTEXTURE_H__424B8540_49CB_11D2_8AA0_00A0C91916A8__INCLUDED_)
#define AFX_DRGTEXTURE_H__424B8540_49CB_11D2_8AA0_00A0C91916A8__INCLUDED_


#define DRGTEXTR_HAS_ALPHA			0x00000001
#define DRGTEXTR_TRANSPARENTWHITE	0x00000002
#define	DRGTEXTR_TRANSPARENTBLACK	0x00000004
#define DRGTEXTR_32BITSPERPIXEL		0x00000008
#define DRGTEXTR_RENDERTARGET		0x00000010 //create texture w. DDSCAPS_3DDEVICE, to render to it.  		
#define DRGTEXTR_FORCEINAGP			0x00000020 //Specifies to create the texture in agp memory
//some flags added for embossing and bump mapping
#define DRGTEXTR_CREATEINVHEIGHTMAP 0x00000040 //will create a texture which is (0.5 - src texture)
#define DRGTEXTR_CREATEBUMPMAP_DUDV 0x00000080 //will create with DUDV caps bits set


#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class CDRGTexture  
{
public:
	//member vars
	HBITMAP              hbmBitmap;      // Windows bitmap containing texture image
	HBITMAP				 hbmAlphaBitmap; // Windows bitmap containing alpha image
    LPDIRECTDRAWSURFACE4 pddsSurface;    // DDraw Surface of the texture
    LPDIRECT3DTEXTURE2   ptexTexture;    // Direct3D texture for the texture
    DWORD                dwStage;        // Texture stage (for multi-textures)
    TCHAR                strName[80];    // Filename of assoc'd bitmap
	TCHAR                strAlphaName[80];  // Filename of bitmap with alpha channel

	bool                 bHasAlpha;		//used to signify 4 or 8 bit alpha plane.	
	
	DWORD                dwFlags;
	
	CDRGTexture();

	// CreateTexture and variants: These functions create one or more windows bitmaps, 
	// either from a file or from scratch. The Restore() function is the one that 
	// actually creates a D3DTexture function

	// Most basic version. Loads a bmp file into an Hbitmap.
	HRESULT CreateTexture( TCHAR* strTextureName, 
						   DWORD dwTextureStage=0L, 
						   DWORD dwTextureFlags=0L );

	// Overloaded member function -loads a texture with a 2nd bmp for the alpha channel
	HRESULT CreateTexture( TCHAR* strTextureName,
						   TCHAR* strAlphaTextureName,							
						   DWORD dwTextureStage=0L, 
						   DWORD dwTextureFlags=0L );

	// Overloaded function to create a texture from a buffer of data 
	//  If pBits is NULL, a blank texture is created 	
	HRESULT CreateTexture( CDRGWrapper *pWrapper,
						   TCHAR*	strTextureName,
						   DWORD	iHres,
						   DWORD	iVres,
						   DWORD	dwBpp, 
						   DWORD dwTextureStage=0L, 
						   DWORD dwTextureFlags=0L,
						   DWORD	*pBits=NULL);

	// Restore() destroys the current device texture and creates a new one from the
	// hbitmap and flags member vars, and device info. (Be aware that even if the 
	// Restore worked once, it mail fail later. For example if you restore after 
	// switching to a device with different caps, or switch to a higher res and run 
	// out of Vidmem)
	HRESULT Restore(LPDIRECT3DDEVICE3 pd3dDevice );

	~CDRGTexture();

private:
	//hidden member functions
	HRESULT			LoadTextureImage( );
	HRESULT			FindTextureFile( TCHAR* strFilename, 
									 TCHAR* strTexturePath, 
									 TCHAR* strFullPath );
	
	HRESULT			CopyBitmapToSurface( LPDIRECTDRAWSURFACE4 pddsTarget, 
										 HBITMAP hbmBitmap,
										 HBITMAP hbmAlphaBitmap,
										 DWORD dwFlags );
	
	HRESULT			RestoreFromBitmap(LPDIRECT3DDEVICE3 pd3dDevice );
	LPDIRECTDRAW4	GetDirectDrawFromDevice( LPDIRECT3DDEVICE3 pd3dDevice );

};

#endif // !defined(AFX_DRGTEXTURE_H__424B8540_49CB_11D2_8AA0_00A0C91916A8__INCLUDED_)
