// Main.cpp
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
//  Implementation of Winmain and message handler. Some stuff related to the embossing 
//  demo is in here (help text, etc)
//
// ------------------------------------------------------------------------------------
//
// Authors: Dean Macri, Kim Pallister - Intel Developer Relations & Engineering Group
//
///////////////////////////////////////////////////////////////////////////////////////

#include "DRGShell.h"
#include "main.h"

#pragma comment(lib,"user32")
#pragma comment(lib,"gdi32")
#pragma comment(lib,"ddraw")
#pragma comment(lib,"dxguid")
#pragma comment(lib,"winmm")

///////////
// Globals
HINSTANCE g_hInst = NULL;       // Instance of this app
HWND g_hWnd  = NULL;			// Handle of the window

int g_iDriverNum = -1;
int g_iDrivers;
int g_iModes;

// Frame rate stuff 
int g_iFrameCount = 0;
DWORD g_dwLastTime = 0;
DWORD g_dwLastFPSTime = 0;
DWORD g_dwCurTime = 0;
DWORD g_dwFps = 0;

bool g_bActive;				// flag to track whether app is active
bool g_bFullscreen;			// flag for toggling full-screen off and on
bool g_bShowSampleHelp;

//globals initing classes
CDRGWrapper *g_pDRGWrapper;		// the DRG wrapper class
CWater *g_pWater;             // the app class where you stick your app-specific stuff
CDRGTexture *g_pCFontTexture;	// instance of the texture class that will hold the font texture
								// used by the text class below.
CDRGText *g_pCMainText;			// instance of the text class that will hold all the strings printed
								// by main.cpp on every frame (for now, that is FPS)
CDRGText *g_pCTitleText;			// instance of the text class that will hold all the strings printed
								// by main.cpp on every frame (for now, that is FPS)
CDRGText *g_pCHelpText;			// instance of the text class that will hold all the strings printed
								// by main.cpp on every frame (for now, that is FPS)


// Functions

// Handles all the windows messages we might receive
long FAR PASCAL WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HRESULT hr;
	int i, iID;

    // Handle messages
    switch (message) 
	{
		// We'll place a message handler here for keypresses until we add some DIrectInput stuff
		case WM_KEYDOWN:			
			switch(wParam) 
			{
				case 'H':
					g_bShowSampleHelp = !g_bShowSampleHelp;
					break;
				case 'X':	//place your code here for what to do if X is pressed (exit for now).
					DestroyWindow( hWnd );
					break;

				case 'F':	// Toggle between full screen and windowed
                    if( g_bActive && g_pDRGWrapper && g_pWater )
					{
						g_bActive = false;
						g_bFullscreen = !g_bFullscreen;
						hr = g_pDRGWrapper->RefreshMode( g_bFullscreen );
						if( FAILED( hr ) )
						{
							DisplayErrorMessage( hr );
							DestroyWindow( hWnd );
						}
						else if( g_pDRGWrapper->Initialized() )
						{
                            hr = g_pWater->TargetChanged();
							g_pCFontTexture->Restore(g_pDRGWrapper->m_pD3DDevice);
							if( FAILED( hr ) )
							{
								DisplayErrorMessage( hr );
								DestroyWindow( hWnd );
							}
							else
								g_bActive = true;
						}
					}
				break;
				
			}
			//now that keypress handled here, pass to the app class
			g_pWater->KeyDown(wParam);
			break;		
		case WM_KEYUP:
			//In this file, we aren't using any key releases, but we'll call 
			// the KeyUp in the app class in case the user needs it
			g_pWater->KeyUp(wParam);
			break;

		//We're not using the mouse here, but we'll pass it to the app
		case WM_MOUSEMOVE:
			if( (g_pDRGWrapper) && (g_pDRGWrapper->Initialized()) )
			{
				g_pWater->MouseMove(wParam,lParam);
			}
			break;

		case WM_LBUTTONDOWN:
			if( (g_pDRGWrapper) && (g_pDRGWrapper->Initialized()) )
			{
				g_pWater->MouseLeftDown(wParam,lParam);
			}
			break;
		
		case WM_LBUTTONUP:
			if( (g_pDRGWrapper) && (g_pDRGWrapper->Initialized()) )
			{
				g_pWater->MouseLeftUp(wParam,lParam);
			}
			break;

		case WM_RBUTTONDOWN:
			if( (g_pDRGWrapper) && (g_pDRGWrapper->Initialized()) )
			{
				g_pWater->MouseRightDown(wParam,lParam);
			}
			break;
		
		case WM_RBUTTONUP:
			if( (g_pDRGWrapper) && (g_pDRGWrapper->Initialized()) )
			{
				g_pWater->MouseRightUp(wParam,lParam);
			}
			break;
		case WM_COMMAND:
			//
			// Because the menus are dynamically created, we have to check
			// the drivers and modes individually
			//
			iID = LOWORD( wParam );
			if( (iID >= IDM_DEVICE_DEVICE0) && (iID < IDM_DEVICE_DEVICE0 + g_iDrivers ) )
			{
				if( iID != g_iDriverNum + IDM_DEVICE_DEVICE0 )
				{
					g_iDriverNum = iID - IDM_DEVICE_DEVICE0;

					HMENU hMenu = GetMenu( g_hWnd );
					hMenu = GetSubMenu( hMenu, 1 );
					for( i=0; i<g_iDrivers; i++ )
						CheckMenuItem( hMenu, IDM_DEVICE_DEVICE0+i, MF_BYCOMMAND | ((g_iDriverNum == i) ? MF_CHECKED : MF_UNCHECKED ) );
					//gray out devices that don't do bump mapping
					for( i=0; i<g_iDrivers; i++ )
					{
						D3DDriver *pDriver = g_pDRGWrapper->GetDriver( i );
						if (pDriver->HALDesc.dwFlags) //then HW
						{
							if (!(pDriver->HALDesc.dwTextureOpCaps & D3DTEXOPCAPS_BUMPENVMAP))
								EnableMenuItem(hMenu, i,MF_GRAYED);
						}
						else
						{
						if (!(pDriver->HELDesc.dwTextureOpCaps & D3DTEXOPCAPS_BUMPENVMAP))
							EnableMenuItem(hMenu, i,MF_GRAYED);
						}
					}

                    if( g_bActive && g_pDRGWrapper && g_pWater )
					{
						g_bActive = false;
						hr = g_pDRGWrapper->m_pD3D->EvictManagedTextures();
						hr = g_pDRGWrapper->ChangeDriver( g_iDriverNum );
						if( FAILED( hr ) )
						{
							DisplayErrorMessage( hr );
							DestroyWindow( hWnd );
						}
						else if( g_pDRGWrapper->Initialized() )
						{
                            hr = g_pWater->TargetChanged();
							g_pCFontTexture->Restore(g_pDRGWrapper->m_pD3DDevice);
							if( FAILED( hr ) )
							{
								DisplayErrorMessage( hr );
								DestroyWindow( hWnd );
							}
							else
								g_bActive = true;
						}
					}
				}
			}
			
			break;


			//
			// Don't let the window get too small
			//
        case WM_GETMINMAXINFO:
            ((MINMAXINFO*)lParam)->ptMinTrackSize.x = 100;
            ((MINMAXINFO*)lParam)->ptMinTrackSize.y = 100;
            break;

		case WM_ACTIVATEAPP:
			// Determine whether app is being activated or not
			g_bActive = wParam ? true : false;

			if (g_bActive) 
			{
				// Hide the cursor if we are active
				//while (ShowCursor(FALSE) > 0) { };
			} 
			else 
			{
				//ShowCursor(TRUE);                
			}
			break;

		case WM_MOVE:
			if( g_bActive && g_pDRGWrapper )
			{
				g_pDRGWrapper->MoveWindow();
			}
			break;
		case WM_SIZE:
			if( (wParam == SIZE_MAXHIDE) || (wParam == SIZE_MINIMIZED ) )
				g_bActive = false;
			else
				g_bActive = true;

            if( g_bActive && g_pDRGWrapper && g_pWater && !g_bFullscreen )
			{
				g_bActive = false;
				//if the device wasn't trashed doing a fullscreen->window change
				if (g_pDRGWrapper->m_pD3D)
					hr = g_pDRGWrapper->m_pD3D->EvictManagedTextures();

				hr = g_pDRGWrapper->RefreshMode( false );
				if( FAILED( hr ) )
				{
					DisplayErrorMessage( hr );
					DestroyWindow( hWnd );
				}
				else if( g_pDRGWrapper->Initialized() )
				{
                    hr = g_pWater->TargetChanged();

					if( FAILED( hr ) )
					{
						DisplayErrorMessage( hr );
						DestroyWindow( hWnd );
					}
					else
					{
						g_bActive = true;
						g_pCFontTexture->Restore(g_pDRGWrapper->m_pD3DDevice);
					}

				}
			}
			break;

		case WM_CLOSE:
			DestroyWindow(g_hWnd);
			break;
		case WM_DESTROY:
			//
			// Destroy the stuff we created
			//
            if( g_pWater ) 
			{
                g_pWater->DestroyWorld();
                delete g_pWater;
                g_pWater = NULL;
			}
			
			if( g_pDRGWrapper )
			{
				delete g_pDRGWrapper;
				g_pDRGWrapper = NULL;
			}

            if( g_pCFontTexture )
			{
				delete g_pCFontTexture;
				g_pCFontTexture = NULL;
			}
			
			

			// And finally get out for good.
			PostQuitMessage(0);
			break;
    }
    
    return DefWindowProc(hWnd, message, wParam, lParam);
}


int FAR PASCAL WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow)
{
	BOOL bGotMessage;
    MSG msg;
	HRESULT hr;
	
    // Set global handle
    g_hInst = hInst;
	
	// Initialize the window required by Windows
    if (!InitWindow(hInst, nCmdShow)) 
	{
		MessageBox( NULL, "Couldn't create a window!", WINDOWNAME, MB_ICONEXCLAMATION | MB_OK );
		return 1;
	}

	//
	// Allocate a CDRGWrapper -- this will enumerate all the D3D devices available
	//
	g_pDRGWrapper = new CDRGWrapper( g_hWnd );

	// Since the D3DWrapper Constructor did the driver enumeration, we will have it select
	// the best default driver, and then init the default mode.
	// I would document the many amazing ways you can do the above, but.....

	//
	// Add the available devices to the device menu
	//
	g_iDrivers = g_pDRGWrapper->GetNumDrivers();
	int i;
	D3DDriver *pDriver;
	MENUITEMINFO mii;

	HMENU hMenu = GetMenu( g_hWnd );
	hMenu = GetSubMenu( hMenu, 1 ); // Get the second sub-menu (Assumes File then Device)
	DeleteMenu( hMenu, IDM_DEVICE_DEVICE0, MF_BYCOMMAND ); // Delete the first menu item

	memset( &mii, 0, sizeof( MENUITEMINFO ) );
	mii.cbSize = sizeof( MENUITEMINFO );
	mii.fMask = MIIM_CHECKMARKS | MIIM_ID | MIIM_STATE | MIIM_TYPE;
	mii.fType = MFT_STRING;
	mii.fState = MFS_ENABLED | MFS_UNCHECKED | MFS_UNHILITE;
 
	for( i=0; i<g_iDrivers; i++ )
	{
		pDriver = g_pDRGWrapper->GetDriver( i );
		mii.wID = IDM_DEVICE_DEVICE0 + i;
		mii.dwTypeData = pDriver->pMenuDescription;
		InsertMenuItem( hMenu, i, TRUE, &mii );
	}

	//gray out devices that don't do bump mapping
	for( i=0; i<g_iDrivers; i++ )
	{
		pDriver = g_pDRGWrapper->GetDriver( i );
		if (pDriver->HALDesc.dwFlags) //then HW
		{
			if (!(pDriver->HALDesc.dwTextureOpCaps & D3DTEXOPCAPS_BUMPENVMAP))
				EnableMenuItem(hMenu, i, MF_BYPOSITION | MF_GRAYED);
		}
		else
		{
			if (!(pDriver->HELDesc.dwTextureOpCaps & D3DTEXOPCAPS_BUMPENVMAP))
				EnableMenuItem(hMenu, i, MF_BYPOSITION | MF_GRAYED);
		}
	}

  


	//
	// Pick the first  device that fits the requirements (HW/SW, AGP) the Appwizard
	// user specified
	//
	
	bool bUseHardware = true;

	bool bRequireAGP = false;
		
	bool bUseBumpMapping = true;

	// This whole init nested if was remmed out because my initial idea was to at least do the
	// render-to-texture if I couldn't do bump mapping. Unfortunately, a lot of hardware improperly
	// OK's a call to CreateSurface when BOTH the Texture and 3DDevice flags are specified, even
	// though they don't support render-to-texture. The nested if was replaced with one that
	// just requires a driver that supports bump mapping

	/*
	if( FAILED( hr = g_pDRGWrapper->GetBestDriver( g_iDriverNum, bUseHardware, bRequireAGP, REQUIREBUMPMAPPING | REQUIRETARGETTEXTURE ) ) )
	{
		//bump mapping may not be supported
		if (FAILED( hr = g_pDRGWrapper->GetBestDriver( g_iDriverNum, bUseHardware, bRequireAGP, REQUIRETARGETTEXTURE) ) )
		{
			//if this failed, then either we have no HW, or HW that can't render to texture.
			// We will revert to sw. no need to check for the target texture flag with SW renderer
			if (FAILED( hr = g_pDRGWrapper->GetBestDriver( g_iDriverNum, false, false, REQUIREBUMPMAPPING) ) )
			{
				//if this failed, then we likely are on a SW-only system with the ref-rast disabled
				// we will enumerate, but should warn the user
				if (FAILED( hr = g_pDRGWrapper->GetBestDriver( g_iDriverNum, false, false) ) )
				{

					//DisplayErrorMessage( hr );
					MessageBox( NULL, 
						"No D3D device found. Exiting.",
						WINDOWNAME, MB_OK );
					return 0;
				}
				else
				{
					//should warn user here - SW only, no bump mapping
					bUseBumpMapping = false;
					MessageBox( NULL, 
						"No Bump Mapping device found. Try enabling Reference Rasterizer",
						WINDOWNAME, MB_OK );
					
				}
				
			}
			else
			{
				//should warn user here - no suitable hw found, reverting to sw device that supports bump mapping
				MessageBox( NULL, 
						"No render-to-texture capable HW found, reverting to SW",
						WINDOWNAME, MB_OK );
			}
		}
		else
		{
			//bump mapping not supported, but using HW anyway. Should we warn that render to texture may not work
			MessageBox( NULL, 
					"HW Device appears to support render-to-texture, but not bump mapping. Use Reference Rasterizer to see bump mapping",
					WINDOWNAME, MB_OK );//found a device with all that we need.
		}
	}
	else
	{
		MessageBox( NULL, 
					"HW Device appears to support render-to-texture and bump mapping. Use Reference Rasterizer if problems appear",
					WINDOWNAME, MB_OK );//found a device with all that we need.
	}

	*/
	
	if( FAILED( hr = g_pDRGWrapper->GetBestDriver( g_iDriverNum, bUseHardware, bRequireAGP, REQUIREBUMPMAPPING | REQUIRETARGETTEXTURE ) ) )
	{
		//if this failed, then either we have no HW, or HW that can't render to texture.
		// We will revert to sw. no need to check for the target texture flag with SW renderer
		if (FAILED( hr = g_pDRGWrapper->GetBestDriver( g_iDriverNum, false, false, REQUIREBUMPMAPPING) ) )
		{
			//if this failed, then we likely are on a SW-only system with the ref-rast disabled
			// we will enumerate, but should warn the user
			if (FAILED( hr = g_pDRGWrapper->GetBestDriver( g_iDriverNum, false, false) ) )
			{

				//DisplayErrorMessage( hr );
				MessageBox( NULL, 
					"No D3D device found. Exiting.",
					WINDOWNAME, MB_OK );
					return 0;
			}
			else
			{
				//should warn user here - SW only, no bump mapping
				bUseBumpMapping = false;
				MessageBox( NULL, 
					"No Bump Mapping device found. Render to texture will still work. Try enabling Reference Rasterizer",
					WINDOWNAME, MB_OK );
				//normally, we would have greyed out this menu item. In this case we will reenable it
				EnableMenuItem( hMenu, IDM_DEVICE_DEVICE0+g_iDriverNum, MF_BYCOMMAND | MF_ENABLED  );
			}
		}
		else
		{
			//should warn user here - no suitable hw found, reverting to sw device that supports bump mapping
			MessageBox( NULL, 
					"SW Device supporting bump mapping was found.",
					WINDOWNAME, MB_OK );
		}
	}
	else
	{
		MessageBox( NULL, 
					"HW Device appears to support render-to-texture and bump mapping. Use Reference Rasterizer if problems appear",
					WINDOWNAME, MB_OK );//found a device with all that we need.
	}

	
	
	// Check the menu item for the driver
	//
	CheckMenuItem( hMenu, IDM_DEVICE_DEVICE0+g_iDriverNum, MF_BYCOMMAND | MF_CHECKED  );

	//
	// Get the modes available so we can setup the menu -- DON'T delete the
	// pModes pointer returned by this function.  It will be cleaned up by the
	// wrapper class
	//
	D3DFSMode *pModes;
	if( FAILED( hr = g_pDRGWrapper->EnumModes( g_iDriverNum, &g_iModes, &pModes ) ) )
	{
		DisplayErrorMessage( hr );
		return 0;
	}
	//
	// Add the modes to the menu
	//
	hMenu = GetMenu( g_hWnd );
	hMenu = GetSubMenu( hMenu, 2 ); // Get the third sub-menu (Assumes File then Device then Screen)
	DeleteMenu( hMenu, IDM_MODE_FULL0, MF_BYCOMMAND ); // Delete the first menu item

	for( i=0; i<g_iModes; i++ )
	{
		mii.dwTypeData = pModes->szDescription;
		mii.wID = IDM_MODE_FULL0 + i;
		InsertMenuItem( hMenu, i+2, TRUE, &mii );
		pModes = pModes->pNext;
	}

	// Initialize the default mode (primary device, Full Screen, 640x480x16, 1 Back buffer, ZBuffer)
	//
	// some examples of different ways to call init mode:
	//	hr = g_pDRGWrapper->InitMode()  
	//		- all defaults. primary display driver, fullscreen, 640x480,1 backbuffer, zbuffer
	//
	//	hr = g_pDRGWrapper->InitMode( int DriverNumber, Bool Fullscreen,  )  
	//		- specify driver, specify window/fs 
	//
	//	hr = g_pDRGWrapper->InitMode( iDriverNum, bFullscreen, iHres,iVres,iBpp,iBuffers,bHw,false )  
	//		- specify driver, specify window/fs, res, etc. The last flag tells it to fall
	//		  back to the default mode if the requested mode fails.
//	//	hr = g_pDRGWrapper->InitMode( 1, true, 640,480,16,2,true,true )  ;
	
	//
	//	hr = g_pDRGWrapper->InitMode( g_iDriverNum, iMode)  
	//		- specify driver and mode number from the list of enumerated modes. Useful with a menu.
	//
	//
	// KIM/DEAN: We will have to do different versions of this snippet depending on the answers to 
	// questions in the appwizard.

    g_bFullscreen = false;




    if( FAILED( hr = g_pDRGWrapper->InitMode(g_iDriverNum,g_bFullscreen, 640, 480, 16, 1, false) ) )
	{
		DisplayErrorMessage( hr );
		return 0;
	}
	//here, I set our fullscreen global, since, depending how InitMode was called, you may not know
	//whether the app is in FS or not.
	g_bFullscreen = g_pDRGWrapper->m_bFullscreen;


	//
	// Initialize the world
	//
    g_pWater = new CWater;
    g_pWater->InitWorld( g_pDRGWrapper );

	//
	// Initialize the Text class, and a font texture for it.
	// (have to cast the second param to dword because of the overloaded fcn)
	//
	g_pCFontTexture = new CDRGTexture;
	if (g_pCFontTexture->CreateTexture(FONTFILENAME, (DWORD)0,DRGTEXTR_TRANSPARENTBLACK) == S_OK)
	//if (g_pCFontTexture->CreateTexture(FONTFILENAME, (DWORD)0,DRGTEXTR_TRANSPARENTWHITE) == S_OK)
		g_pCFontTexture->Restore(g_pDRGWrapper->m_pD3DDevice);

	g_pCMainText = new CDRGText;
	g_pCMainText->Init( g_pDRGWrapper );
	g_pCMainText->SetFontTexture( g_pCFontTexture );

	g_pCTitleText = new CDRGText;
	g_pCTitleText->Init( g_pDRGWrapper );
	g_pCTitleText->SetFontTexture( g_pCFontTexture );

	g_pCHelpText = new CDRGText;
	g_pCHelpText->Init( g_pDRGWrapper );
	g_pCHelpText->SetFontTexture( g_pCFontTexture );



	int mainstringarray[2];
	int titlestringarray[2];
	int helpstringarray[12];

	//This will be the 2 strings for the frame counter. I will keep the length at
	// 3 digits, and then not have to reallocate memory when modifying
	g_pCMainText->CreateString("FPS:", 0.0f, 0.9f, 0.05f, 0.05f, 50, &mainstringarray[0]);
	g_pCMainText->CreateString("XXX", 0.2f, 0.9f, 0.05f, 0.05f, 50, &mainstringarray[1]);

	//The help and title text are store separately, so I can toggle them, making separate calls to draw each
	g_pCTitleText->CreateString("Intel Bump Map sample", 0.0f, 0.02f, 0.04f, 0.06f, 50, &titlestringarray[0]);
	g_pCTitleText->CreateString("(Press H for Help)", 0.0f, 0.8f, 0.02f, 0.03f, 50, &titlestringarray[1]);

    float fYPos = 0.05f;

	g_pCHelpText->CreateString("C - Draw Polygon Cube",						0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[0]); fYPos += 0.05f;
	g_pCHelpText->CreateString("I - Draw Fish/Bird",							0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[1]); fYPos += 0.05f;
    g_pCHelpText->CreateString("W - Draw Wireframe",							0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[2]); fYPos += 0.05f;
	g_pCHelpText->CreateString("T - Draw Text on buffers",						0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[3]); fYPos += 0.05f;
	g_pCHelpText->CreateString("B - Toggle Bump Mapping",						0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[4]); fYPos += 0.05f;
	g_pCHelpText->CreateString("U - Toggle UV animated/Procedural bump map",	0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[5]); fYPos += 0.05f;
	g_pCHelpText->CreateString("V - Toggle waves on procedural bump map",		0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[6]); fYPos += 0.05f;
	g_pCHelpText->CreateString("R - Toggle rain on procedural bump map",		0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[7]); fYPos += 0.05f;
	g_pCHelpText->CreateString("P - Pause animation",							0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[8]); fYPos += 0.05f;
    g_pCHelpText->CreateString("X - Exit",										0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[9]); fYPos += 0.05f;
    g_pCHelpText->CreateString("F - Toggle Fullscreen",							0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[10]); fYPos += 0.05f;
    g_pCHelpText->CreateString("H - Toggle Help",								0.0f, fYPos, 0.02f, 0.03f, 50, &helpstringarray[11]); fYPos += 0.05f;
	
	
	////////////////
	// Main loop! //
	////////////////
	PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE );
    while( msg.message != WM_QUIT ) 
	{ 
		
		// 1. Peek at the Messages	(use GetMessage if we're not active)
		if( g_bActive )
			bGotMessage = PeekMessage( &msg, NULL, 0, 0, PM_REMOVE );
		else
			bGotMessage = GetMessage( &msg, NULL, 0, 0 );

		if( bGotMessage )  // only process 1 message per frame
		{	
            TranslateMessage(&msg);
            DispatchMessage(&msg);
		}
		
		//update the timer vars
		
		g_dwLastTime = g_dwCurTime;
		g_dwCurTime = timeGetTime();		
		if( g_pDRGWrapper )
			g_pDRGWrapper->m_fFrameTime = (float)(g_dwCurTime - g_dwLastTime) / 1000.0f;
		else
			g_dwCurTime = timeGetTime();		

		
		// 2. Update the timer info. How much time has expired since the last frame.
		if (++g_iFrameCount == TIMECHECKINTERVAL) // We check the time every ten frames 
		{
			// We are only doing time check every 10 frames

			
			
			
			//calculate the frame rate.
			g_dwFps= (g_iFrameCount*1000) / (g_dwCurTime-g_dwLastFPSTime);

            char string[10];
            sprintf( string, "%03d", g_dwFps );
			g_pCMainText->ModifyStringContent(mainstringarray[1],string);

			

			//now reset counters for next time we come round.
			g_dwLastFPSTime = g_dwCurTime;
			g_iFrameCount = 0; //reset the frame counter
		} //end if

        if( g_bActive && g_pWater && g_pDRGWrapper )
		{
			if( g_pDRGWrapper->Initialized() )
			{
				//g_iCheezyInput = 0;
				// 3. Do the Application loop. 
				g_pDRGWrapper->Clear( D3DCLEAR_TARGET );

				// I moved updateworld to get better parallelism with the rendering, 
				//  since the last thing I do in UpdateWorld() is lock a texture surface
                //g_pWater->UpdateWorld(); 

				g_pDRGWrapper->StartFrame(); 

                g_pWater->RenderWorld(); //Application calls drawprim, d3drm render, or whatever.
				
				//Draw Text
				g_pCMainText->DrawText(); 
				if (g_bShowSampleHelp)
					g_pCHelpText->DrawText();
				else
					g_pCTitleText->DrawText();

				g_pWater->UpdateWorld(); //Step function to update timers, move objects, etc
				
				g_pDRGWrapper->EndFrame(); //Does Endscene and flip/blit
			
			}
		}
	} 

    return msg.wParam;

} //end Winmain



// Initializes all that windows junk, creates class then shows main window
BOOL InitWindow(HINSTANCE hInst, int nCmdShow)
{
	WNDCLASS    wndClass;

    // Fill out WNDCLASS info
    wndClass.style              = CS_HREDRAW | CS_VREDRAW;
    wndClass.lpfnWndProc        = WndProc;
    wndClass.cbClsExtra         = 0;
    wndClass.cbWndExtra         = 0;
    wndClass.hInstance          = hInst;
    wndClass.hIcon              = LoadIcon(hInst, "AppWizard");
    wndClass.hCursor            = LoadCursor(NULL, IDC_ARROW);
    wndClass.hbrBackground      = (HBRUSH)GetStockObject(BLACK_BRUSH);
    wndClass.lpszMenuName       = MAKEINTRESOURCE( IDR_MENU1 );
    wndClass.lpszClassName      = CLASSNAME;
    
	// We only want to run one at a time anyway, so just quit if another is running
    if (!RegisterClass(&wndClass)) 
		return FALSE;

    // Create a window
    g_hWnd = CreateWindowEx(WS_EX_APPWINDOW,
                            CLASSNAME, 
                            WINDOWNAME,
                            WS_OVERLAPPEDWINDOW,
                            CW_USEDEFAULT, CW_USEDEFAULT,
							320, 240,
                            NULL,
                            NULL,
                            hInst,
                            NULL);

    // Return false if window creation failed
    if (!g_hWnd) 
		return FALSE;
    
    // Show the window
    ShowWindow(g_hWnd, SW_SHOWNORMAL);

    // Update the window
    UpdateWindow(g_hWnd);

	return TRUE;
}



