// Water.h
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
// Water.h: implementation the Water class.
// This class is the "blank" class created by the DRG 3D Appwizard, where you will put your stuff
//
// ------------------------------------------------------------------------------------
//
// Author: Kim Pallister - Intel Developer Relations & Engineering Group
//
////////////////////////////////////////////////////////////////////////////////////////

#include "DRGShell.h"

inline DWORD FLOATtoDWORD( FLOAT f ) { return *((DWORD*)&f); }

// -----------------------------------------------------------------------
// CWater::CWater() 
// The Water class constructor initializes some of it's member vars (which
// are instances of the DRGMatrix class
// -----------------------------------------------------------------------
CWater::CWater()
{

	m_dwFrameCount = 0;

	m_vCameraVector = DRGVector( 0, 0, 0 ) - DRGVector( 0, 1, -5 ); 

	m_MxWorld.SetIdentity();
	//set the camera at 0,0,-5; the target at 0,0,0 and the up vector at 0,1,0
	m_MxView.SetView( DRGVector( 0, 0, -5 ), DRGVector( 0, 0, 0 ), DRGVector( 0, 1, 0 ) );
	//set the projection matrix to 1.57 field of view (radians), 1.33 aspect ratio 
	// (640x480), //the near clip plane to 1 and the far to clip plane 100
	m_MxProjection.SetProjection( 1.05f, 0.5f, 1.0f, 100.0f );

	m_pEnvMapTexture = NULL;
	m_pAboveWaterTexture = NULL;
	m_pBelowWaterTexture = NULL;

	m_pEnvMapTextureBackground = NULL;
	m_pAboveWaterTextureBackground = NULL;
	m_pBelowWaterTextureBackground = NULL;

	m_pBirdTexture = NULL;
	m_pFishTexture = NULL;

	m_pBumpMapProcTexture = NULL;
	m_pBumpMapAnimTexture = NULL;

	
	m_bDrawText			= false;
	m_bDrawBumps		= true;
	m_bDrawCube			= true;
	m_bDrawFish			= true;
	m_bDrawWater		= true;
	m_bDrawWireframe	= false;
	m_bPauseAnim		= false;
	
	m_bProcedural		= true;
	m_bProceduralRain	= false;
	m_bProceduralWaves	= true;



	//init sin and cos tables
	for (int i=0;i<256;i++)
	{
		m_iSinTimes32[i] = (int)(32.0 * sin(3.14f*2.0f*((float)(i)/256.0f)));
		m_iCosTimes32[i] = (int)(32.0 * cos(3.14f*2.0f*((float)(i)/256.0f)));
	}
	
}

// -----------------------------------------------------------------------
// CWater::~CWater()
// destructor does nothing for now.
// -----------------------------------------------------------------------
CWater::~CWater()
{

}

// -----------------------------------------------------------------------
// HRESULT CWater::InitWorld( CDRGWrapper *pWrapper )
//
// -----------------------------------------------------------------------
HRESULT CWater::InitWorld( CDRGWrapper *pWrapper )
{
	HRESULT hr;

	if (pWrapper->m_bHardware)
	{
		if (pWrapper->m_pCurrentDriver->HALDesc.dwTextureOpCaps & D3DTEXOPCAPS_BUMPENVMAP)
			m_bDeviceDoesBumpMapping = true;
		else
			m_bDeviceDoesBumpMapping = false;
	}
	else
	{
		if (pWrapper->m_pCurrentDriver->HELDesc.dwTextureOpCaps & D3DTEXOPCAPS_BUMPENVMAP)
			m_bDeviceDoesBumpMapping = true;
		else
			m_bDeviceDoesBumpMapping = false;
	}



	// Make local copy of the pointer to the wrapper class
	m_pWrapper = pWrapper;

	//Create 3 viewports for our different points of view
	hr = m_pWrapper->m_pD3D->CreateViewport(&m_D3DViewportEnvMap, NULL);
	hr = m_pWrapper->m_pD3D->CreateViewport(&m_D3DViewportBelowWater, NULL);
	hr = m_pWrapper->m_pD3D->CreateViewport(&m_D3DViewportAboveWater, NULL);

	// This function sets up stuff and restores textures after changing 
	// the device. We call it the first time in to the app.
	TargetChanged();

	// First we will set up some sample data to draw the World Axis & plane
	float scale = 1.0f;

	//we are including a sample cube at 0,-1,0 
	SetupSampleCubeVerts(scale * 0.75f); 

	
	//should probably check this on the fly from the texture info, but it's hardcoded for now
	texturesize = 512;

	//environment buffer and backup
	m_pEnvMapTexture = new CDRGTexture;
	if (m_pEnvMapTexture->CreateTexture("backgnd.bmp", (DWORD)0,DRGTEXTR_RENDERTARGET) == S_OK)
		m_pEnvMapTexture->Restore(pWrapper->m_pD3DDevice);
	
	m_pEnvMapTextureBackground = new CDRGTexture;
	if (m_pEnvMapTextureBackground->CreateTexture("backgnd.bmp", (DWORD)0,DRGTEXTR_RENDERTARGET) == S_OK)
		m_pEnvMapTextureBackground->Restore(pWrapper->m_pD3DDevice);
	
	//above water buffer and backup
	m_pAboveWaterTexture = new CDRGTexture;
	if (m_pAboveWaterTexture->CreateTexture("backgnd.bmp", (DWORD)0,DRGTEXTR_RENDERTARGET) == S_OK)
		m_pAboveWaterTexture->Restore(pWrapper->m_pD3DDevice);

	m_pAboveWaterTextureBackground = new CDRGTexture;
	if (m_pAboveWaterTextureBackground->CreateTexture("backgnd.bmp", (DWORD)0,DRGTEXTR_RENDERTARGET) == S_OK)
		m_pAboveWaterTextureBackground->Restore(pWrapper->m_pD3DDevice);

	//below water buffer and backup
	m_pBelowWaterTexture = new CDRGTexture;
	if (m_pBelowWaterTexture->CreateTexture("seabed.bmp", (DWORD)0,DRGTEXTR_RENDERTARGET) == S_OK)
		m_pBelowWaterTexture->Restore(pWrapper->m_pD3DDevice);

	m_pBelowWaterTextureBackground = new CDRGTexture;
	if (m_pBelowWaterTextureBackground->CreateTexture("seabed.bmp", (DWORD)0,DRGTEXTR_RENDERTARGET) == S_OK)
		m_pBelowWaterTextureBackground->Restore(pWrapper->m_pD3DDevice);

	//cheezy artwork for fish and bird
	m_pFishTexture = new CDRGTexture;
	if (m_pFishTexture->CreateTexture("fish.bmp", "fish_a.bmp",(DWORD)0) == S_OK)
		m_pFishTexture->Restore(pWrapper->m_pD3DDevice);
	m_pBirdTexture = new CDRGTexture;
	if (m_pBirdTexture->CreateTexture("bird.bmp", "bird_a.bmp",(DWORD)0) == S_OK)
		m_pBirdTexture->Restore(pWrapper->m_pD3DDevice);

	
	//create the two bump maps we will use (one will be animated, one procedural)

	//this first bit just creates the bump data that we will write to the bump map
	char UVBumpArray[PROCTEXSIZE*PROCTEXSIZE*4];
	InitBumpMapArray(PROCTEXSIZE,PROCTEXSIZE, UVBumpArray);

	m_pBumpMapProcTexture = new CDRGTexture;
	if (m_pBumpMapProcTexture->CreateTexture(pWrapper, "Bump Map", PROCTEXSIZE,PROCTEXSIZE, 16,
									 0, DRGTEXTR_CREATEBUMPMAP_DUDV | DRGTEXTR_FORCEINAGP, NULL) == S_OK)
	{
		if (m_bDeviceDoesBumpMapping)
		{
			m_pBumpMapProcTexture->Restore(pWrapper->m_pD3DDevice);
		}
	}
	
	m_pBumpMapAnimTexture = new CDRGTexture;
	if (m_pBumpMapAnimTexture->CreateTexture(pWrapper, "Bump Map", PROCTEXSIZE,PROCTEXSIZE, 16,
									 0, DRGTEXTR_CREATEBUMPMAP_DUDV | DRGTEXTR_FORCEINAGP, (unsigned long *)UVBumpArray) == S_OK)
	{
		if (m_bDeviceDoesBumpMapping)
		{
			m_pBumpMapAnimTexture->Restore(pWrapper->m_pD3DDevice);
		}
	}
	
	////
	// - set new viewport2's to be used for the next few scenes
	// all the same for now till we start working out geometry...
	m_D3DViewport2EnvMapStruct.dwSize = sizeof(D3DVIEWPORT);
	m_D3DViewport2EnvMapStruct.dwX =			0; 
	m_D3DViewport2EnvMapStruct.dwY =			0;
	m_D3DViewport2EnvMapStruct.dwWidth =		texturesize;  //hardcoded for now, but should change
	m_D3DViewport2EnvMapStruct.dwHeight =		texturesize;  // by adding a size member var to the texture class
	m_D3DViewport2EnvMapStruct.dvClipX =		-1.0f;
	m_D3DViewport2EnvMapStruct.dvClipY =		1.0f;
	m_D3DViewport2EnvMapStruct.dvClipWidth =	2.0f;
	m_D3DViewport2EnvMapStruct.dvClipHeight = 2.0f;
	m_D3DViewport2EnvMapStruct.dvMinZ =		0.0f;
	m_D3DViewport2EnvMapStruct.dvMaxZ =		1.0f;

	m_D3DViewport2BelowWaterStruct.dwSize = sizeof(D3DVIEWPORT);
	m_D3DViewport2BelowWaterStruct.dwX =			0; 
	m_D3DViewport2BelowWaterStruct.dwY =			0;
	m_D3DViewport2BelowWaterStruct.dwWidth =		texturesize;  //hardcoded for now, but should change
	m_D3DViewport2BelowWaterStruct.dwHeight =		texturesize;  // by adding a size member var to the texture class
	m_D3DViewport2BelowWaterStruct.dvClipX =		-1.0f;
	m_D3DViewport2BelowWaterStruct.dvClipY =		1.0f;
	m_D3DViewport2BelowWaterStruct.dvClipWidth =	2.0f;
	m_D3DViewport2BelowWaterStruct.dvClipHeight = 2.0f;
	m_D3DViewport2BelowWaterStruct.dvMinZ =		0.0f;
	m_D3DViewport2BelowWaterStruct.dvMaxZ =		1.0f;
	
	m_D3DViewport2AboveWaterStruct.dwSize = sizeof(D3DVIEWPORT);
	m_D3DViewport2AboveWaterStruct.dwX =			0; 
	m_D3DViewport2AboveWaterStruct.dwY =			0;
	m_D3DViewport2AboveWaterStruct.dwWidth =		texturesize;  //hardcoded for now, but should change
	m_D3DViewport2AboveWaterStruct.dwHeight =		texturesize;  // by adding a size member var to the texture class
	m_D3DViewport2AboveWaterStruct.dvClipX =		-1.0f;
	m_D3DViewport2AboveWaterStruct.dvClipY =		1.0f;
	m_D3DViewport2AboveWaterStruct.dvClipWidth =	2.0f;
	m_D3DViewport2AboveWaterStruct.dvClipHeight = 2.0f;
	m_D3DViewport2AboveWaterStruct.dvMinZ =		0.0f;
	m_D3DViewport2AboveWaterStruct.dvMaxZ =		1.0f;
	
	
	//m_D3DViewportEnvMap->SetViewport2(&m_D3DViewport2EnvMapStruct);
	m_D3DViewport2MainAppStruct.dwSize = sizeof(D3DVIEWPORT);
	
	//hack to stretch the above water texture a bit
	
	m_rectAboveWaterScene.verts[0].tv = 0.3f;
	m_rectAboveWaterScene.verts[1].tv = 0.3f;
	m_rectAboveWaterScene.verts[3].tv = 0.3f;


	//hack to get the edge of the texture to not show wrapping
	m_rectFinalBelow.verts[0].tu2 = 0.05f;
	m_rectFinalBelow.verts[3].tu2 = 0.05f;
	m_rectFinalBelow.verts[5].tu2 = 0.05f;

	m_rectFinalBelow.verts[1].tu2 = 0.95f;
	m_rectFinalBelow.verts[2].tu2 = 0.95f;
	m_rectFinalBelow.verts[4].tu2 = 0.95f;

	m_rectFinalBelow.verts[0].tv2 = 0.05f;
	m_rectFinalBelow.verts[1].tv2 = 0.05f;
	m_rectFinalBelow.verts[3].tv2 = 0.05f;

	m_rectFinalBelow.verts[2].tv2 = 0.95f;
	m_rectFinalBelow.verts[4].tv2 = 0.95f;
	m_rectFinalBelow.verts[5].tv2 = 0.95f;

	////
	m_rectFinalBelow.verts[0].tu = 0.05f;
	m_rectFinalBelow.verts[3].tu = 0.05f;
	m_rectFinalBelow.verts[5].tu = 0.05f;

	m_rectFinalBelow.verts[1].tu = 0.95f;
	m_rectFinalBelow.verts[2].tu = 0.95f;
	m_rectFinalBelow.verts[4].tu = 0.95f;

	m_rectFinalBelow.verts[0].tv = 0.05f;
	m_rectFinalBelow.verts[1].tv = 0.05f;
	m_rectFinalBelow.verts[3].tv = 0.05f;

	m_rectFinalBelow.verts[2].tv = 0.95f;
	m_rectFinalBelow.verts[4].tv = 0.95f;
	m_rectFinalBelow.verts[5].tv = 0.95f;

	////
	m_rectFinalAbove.verts[0].tu = 0.05f;
	m_rectFinalAbove.verts[3].tu = 0.05f;
	m_rectFinalAbove.verts[5].tu = 0.05f;

	m_rectFinalAbove.verts[1].tu = 0.95f;
	m_rectFinalAbove.verts[2].tu = 0.95f;
	m_rectFinalAbove.verts[4].tu = 0.95f;

	m_rectFinalAbove.verts[0].tv = 0.05f;
	m_rectFinalAbove.verts[1].tv = 0.05f;
	m_rectFinalAbove.verts[3].tv = 0.05f;

	m_rectFinalAbove.verts[2].tv = 0.95f;
	m_rectFinalAbove.verts[4].tv = 0.95f;
	m_rectFinalAbove.verts[5].tv = 0.95f;

	//this will set my vertex colors for when I want to blend them together
	m_rectBelowWaterScene.verts[0].color = 0xff000000; 
	m_rectBelowWaterScene.verts[1].color = 0xff000000; 
	m_rectBelowWaterScene.verts[2].color = 0xff8888ff; 
	m_rectBelowWaterScene.verts[3].color = 0xff000000; 
	m_rectBelowWaterScene.verts[4].color = 0xff8888ff; 
	m_rectBelowWaterScene.verts[5].color = 0xff8888ff; 
		
	m_rectAboveWaterScene.verts[0].color = 0xffffffff; 
	m_rectAboveWaterScene.verts[1].color = 0xffffffff; 
	m_rectAboveWaterScene.verts[2].color = 0xff000000; 
	m_rectAboveWaterScene.verts[3].color = 0xffffffff; 
	m_rectAboveWaterScene.verts[4].color = 0xff000000; 
	m_rectAboveWaterScene.verts[5].color = 0xff000000; 

	m_fFishPosX = 0.5f;
	m_fFishPosY = 0.7f;
	m_fBirdPosX = 0.6f;
	m_fBirdPosY = 0.5f;

	memset(m_DripTexBuffer,0,PROCTEXSIZE*PROCTEXSIZE*sizeof(WORD));
	memset(m_ClickBuffer,0,PROCTEXSIZE*PROCTEXSIZE*sizeof(int));

	srand(timeGetTime()); //seed random number generator; we use it for rain

	return S_OK;
}

// -----------------------------------------------------------------------
// HRESULT CWater::DestroyWorld( void )
//
// -----------------------------------------------------------------------
HRESULT CWater::DestroyWorld( void )
{
	return S_OK;
}

// -----------------------------------------------------------------------
// HRESULT CWater::UpdateWorld( DWORD dwTime, int InputFlag )
//
// -----------------------------------------------------------------------
HRESULT CWater::UpdateWorld()
{
	m_dwFrameCount++;
	
	// I should calculate the various view frustums and the like, depending on the view vector
	// but for now it is just hardcoded to show the render-to-texture infrastructure
	//	

	m_rectBelowWaterScene.SetSize(0,0,512,512);
	m_rectAboveWaterScene.SetSize(0,0,512,512);
	
	//flip verts of m_rectAboveWaterScene - this is just temporary way to do it, and will likely get culled by hw
	for (int i=0;i<6;i++)
	{
		m_rectAboveWaterScene.verts[i].tv = 1.0f - m_rectFinalAbove.verts[i].tv;		
	}
	//this just simluates the calculation of where the reflection gets clipped...
	m_rectAboveWaterScene.verts[2].tv = 0.1f;//1.0f;
	m_rectAboveWaterScene.verts[4].tv = 0.1f;//1.0f;
	m_rectAboveWaterScene.verts[5].tv = 0.1f;//1.0f;

	m_rectAboveWaterBackgnd.SetSize(0.0f,0.0f, 512.0f,512.0f);
	m_rectBelowWaterBackgnd.SetSize(0.0f,0.0f, 512.0f,512.0f);
	
	
	float fFinalSplit = (float)m_pWrapper->m_dwRenderHeight * 0.33f; 
	m_rectFinalAbove.SetSize(0.0f,0.0f,  (float)m_pWrapper->m_dwRenderWidth,fFinalSplit);
	m_rectFinalBelow.SetSize(0.0f,fFinalSplit,  (float)m_pWrapper->m_dwRenderWidth,(float)m_pWrapper->m_dwRenderHeight);


	// Animate the fish and bird sprites
	if (!m_bPauseAnim)
	{
		m_fFishPosX += m_pWrapper->m_fFrameTime * 0.2f;
		m_fBirdPosX -= m_pWrapper->m_fFrameTime * 0.5f;
		// Reset them if necesary
		if (m_fFishPosX > 1.0)
			m_fFishPosX = 0.0f;
		if (m_fBirdPosX < 0.0)
			m_fBirdPosX = 1.0f;
	}

	
	m_rectSampleFish.SetSize( (m_fFishPosX-0.2f) * 512.0f,
							  (m_fFishPosY-0.2f) * 512.0f,
							  (m_fFishPosX+0.2f) * 512.0f,
							  (m_fFishPosY+0.2f) * 512.0f);

	m_rectSampleBird.SetSize( (m_fBirdPosX-0.07f) * 512.0f,
							  (m_fBirdPosY-0.1f) * 512.0f,
							  (m_fBirdPosX+0.07f) * 512.0f,
							  (m_fBirdPosY+0.1f) * 512.0f);

	//procedurally do stuff to the bump map
	if (m_bProcedural && m_bDeviceDoesBumpMapping)
	{
		//I am only going to do this if the texture was allocated in AGP mem	
		if (m_pBumpMapProcTexture->dwFlags & DRGTEXTR_FORCEINAGP)
		{
			DDSURFACEDESC2 lpSurfDesc;
			ZeroMemory( &lpSurfDesc, sizeof(DDSURFACEDESC2) );
			lpSurfDesc.dwSize                 = sizeof(DDSURFACEDESC2);
			lpSurfDesc.dwFlags                = 0;
			lpSurfDesc.ddsCaps.dwCaps         = 0;
			lpSurfDesc.ddpfPixelFormat.dwSize = sizeof(DDPIXELFORMAT);
	
			//procedural texture routine.
			// we go through the loop below doing two things:
			//		- generating a procedural wave, whose frequency increases toward the top (to
			//		  fake the appearance of perspective
			//		- checking if our buffer of mouse clicks has a record of a mouse click in that
			//		  spot. If so, we generate the waves moving outward
			///////
			//  clear DripTexBuffer
			memset(m_DripTexBuffer,0,PROCTEXSIZE*PROCTEXSIZE*sizeof(WORD));

			if (m_bProceduralRain && !m_bPauseAnim)
			{
				//set one random raindrop per frame, equivalent to a mouse click
				int xrand = 12 + (rand()%(PROCTEXSIZE-24));
				int yrand = 12 + (rand()%(PROCTEXSIZE-24));

				m_ClickBuffer[yrand * PROCTEXSIZE+xrand] = 24;					
			}

			//now go through a loop for each pixel in the buffers, generating waves in one
			//and propogating mouse click-n-drags as waves in another

			char   *ptmp = (char *)m_CompTexBuffer;
			char   *pDriptmp = (char *)m_DripTexBuffer;
			float oneovertexsize = 1.0f/(float)PROCTEXSIZE;
			char cDu,cDv;
			int index;
		
			for( DWORD y=1; y<PROCTEXSIZE; y++ )
			{
				for( DWORD x=1;x<PROCTEXSIZE; x++ )
				{
					// generate waves
					float freq = (float) ((PROCTEXSIZE-y) + 10)*oneovertexsize;
					//cube freq to fake perspective
					index = (int)((float)(x+64) * freq * freq * freq *64.0f);
					float offset = (float)y*oneovertexsize;
				
					if (!m_bPauseAnim)
						index += (m_dwFrameCount*8)%256;

					//cube offset to fake perspective
					index += (int)(offset * offset * offset);
					index = index%256;
					
					if (m_bProceduralWaves)
					{
						cDu = (char)(m_iCosTimes32[index]);
						cDv = (char)(m_iSinTimes32[index]);
					}
					else
					{
						cDu = (char)0;
						cDv = (char)0;
					}
					
					// this first if is a cheap way to get around checking if the circles 
					// we are plotting are outside the buffer;
					if ((y>12) && y<(PROCTEXSIZE-12))
					{	// if clickbuffer !=0
						if (m_ClickBuffer[y*PROCTEXSIZE+x]!=0)
						{
							int radius = (23 - m_ClickBuffer[y*PROCTEXSIZE+x])/2;
							WORD intensity = (char)(0x0077 - (WORD)(m_ClickBuffer[y*PROCTEXSIZE+x])); //32 - intensity in clkbuff
							intensity = 0x0077;

							// draw a circle in the DripTexBuffer using Besenhan integer
							// circle algorithm (i.e. doesn't handle center at non integer values)
							int cx = 0;
							int cy = radius;
							float f = 5.0f/4.0f - radius;
						
							//take advantage of symmetry, note that the value is a rough approximation of slope,
							// by getting the Du and Dv 'direction' right, but not the quantity
							m_DripTexBuffer[(y*PROCTEXSIZE+x) + (cy*PROCTEXSIZE+cx)] = ((intensity<<8) | intensity); //x y
							m_DripTexBuffer[(y*PROCTEXSIZE+x) + (cx*PROCTEXSIZE+cy)] = ((intensity<<8) | intensity); //y x
							m_DripTexBuffer[(y*PROCTEXSIZE+x) + (cx*PROCTEXSIZE-cy)] = (((64-intensity)<<8) | intensity); //y -x
							m_DripTexBuffer[(y*PROCTEXSIZE+x) + (-cy*PROCTEXSIZE+cx)] = ((intensity<<8) | 64-intensity);//x -y
							m_DripTexBuffer[(y*PROCTEXSIZE+x) + (-cy*PROCTEXSIZE-cx)] = (((64-intensity)<<8) | (64-intensity));//-x-y 
							m_DripTexBuffer[(y*PROCTEXSIZE+x) + (-cx*PROCTEXSIZE-cy)] = (((64-intensity)<<8) | (64-intensity));//-y-x
							m_DripTexBuffer[(y*PROCTEXSIZE+x) + (-cx*PROCTEXSIZE+cy)] = ((intensity<<8) | (64-intensity));//-y x
							m_DripTexBuffer[(y*PROCTEXSIZE+x) + (cy*PROCTEXSIZE-cx)] = (((64-intensity)<<8) | intensity);//-x y

							while (cy>cx)
							{
								if (f<0)
									f += 2.0f * cx + 3.0f;
								else
								{
									f += 2.0f * (cx-cy) + 5.0f;
									cy--;
								}
								cx++;
								//plot rest of points on circle
								m_DripTexBuffer[(y*PROCTEXSIZE+x) + (cy*PROCTEXSIZE+cx)] = ((intensity<<8) | intensity); //x y
								m_DripTexBuffer[(y*PROCTEXSIZE+x) + (cx*PROCTEXSIZE+cy)] = ((intensity<<8) | intensity); //y x
								m_DripTexBuffer[(y*PROCTEXSIZE+x) + (cx*PROCTEXSIZE-cy)] = (((64-intensity)<<8) | intensity); //y -x
								m_DripTexBuffer[(y*PROCTEXSIZE+x) + (-cy*PROCTEXSIZE+cx)] = ((intensity<<8) | 64-intensity);//x -y
								m_DripTexBuffer[(y*PROCTEXSIZE+x) + (-cy*PROCTEXSIZE-cx)] = (((64-intensity)<<8) | (64-intensity));//-x-y 
								m_DripTexBuffer[(y*PROCTEXSIZE+x) + (-cx*PROCTEXSIZE-cy)] = (((64-intensity)<<8) | (64-intensity));//-y-x
								m_DripTexBuffer[(y*PROCTEXSIZE+x) + (-cx*PROCTEXSIZE+cy)] = ((intensity<<8) | (64-intensity));//-y x
								m_DripTexBuffer[(y*PROCTEXSIZE+x) + (cy*PROCTEXSIZE-cx)] = (((64-intensity)<<8) | intensity);//-x y

							} //end while
					
							// intensity depends on clickbuffervalue
							
							// decrement clickbuffer value, unless pause
							if (!m_bPauseAnim)
								m_ClickBuffer[y*PROCTEXSIZE+x] -= 1;
						} //end if clickbuffer non zero
					} //end if within range
					// add clickbuffer value to waves
					// In the U8V8 format, 1st byte is U, 2nd byte is V.
					ptmp[x*2+0] = cDu;// + pDriptmp[x*2+0];
					ptmp[x*2+1] = cDv;// + pDriptmp[x*2+1];
				} //next x
				ptmp += 2*PROCTEXSIZE;
				pDriptmp +=2*PROCTEXSIZE;
			} //next y

			
			//we need to do another pass to add them together
			WORD *ptmp2 = m_CompTexBuffer;
			for( y=1; y<PROCTEXSIZE; y++ )
			{
				for( DWORD x=1;x<PROCTEXSIZE; x++ )
				{
					ptmp2[x] += m_DripTexBuffer[y*128+x];
					//ptmp[x*2+1] += m_DripTexBuffer[y*128+x];
				}
				ptmp2 += PROCTEXSIZE;
			}
			

			//now lock the surface and copy the bits over.
			while( m_pBumpMapProcTexture->pddsSurface->Lock( NULL, &lpSurfDesc, 0, NULL ) ==
				   DDERR_WASSTILLDRAWING );
		
			memcpy(lpSurfDesc.lpSurface,m_CompTexBuffer,PROCTEXSIZE*PROCTEXSIZE*sizeof(WORD));

			m_pBumpMapProcTexture->pddsSurface->Unlock(NULL);
		
		}
		else
		{
			OutputDebugString ("Texture not in AGP, so I won't try locking it");
		}		
	}//end if procedural
	else
	{
		if (m_bDrawBumps)
		{
			//if using the UV animation instead...
			if ((m_pWrapper->m_fFrameTime < 100.0f) && (!m_bPauseAnim) && m_bDrawBumps)
			{	
				for (i=0;i<6;i++)
				{
					m_rectFinalBelow.verts[i].tu += m_pWrapper->m_fFrameTime * 0.2f;
					//m_rectFinalBelow.verts[i].tv += 0.005f;
				}			
			}
		}
		else
		{
			m_rectFinalBelow.verts[0].tu = 0.05f;
			m_rectFinalBelow.verts[1].tu = 0.95f;
			m_rectFinalBelow.verts[2].tu = 0.95f;
			m_rectFinalBelow.verts[3].tu = 0.05f;
			m_rectFinalBelow.verts[4].tu = 0.95f;
			m_rectFinalBelow.verts[5].tu = 0.05f;
		}
	}

	return S_OK;
}

// -----------------------------------------------------------------------
// HRESULT CWater::RenderWorld( void )
//
// -----------------------------------------------------------------------
HRESULT CWater::RenderWorld( void )
{
	HRESULT hr; //unfortunately, I don't do nearly the amount of error checking I should...

	hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, NULL);	
	
	hr = m_pWrapper->m_pD3DDevice->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, TRUE);
	hr = m_pWrapper->m_pD3DDevice->SetRenderState( D3DRENDERSTATE_DITHERENABLE, TRUE );
	hr = m_pWrapper->m_pD3DDevice->SetRenderState( D3DRENDERSTATE_SPECULARENABLE, FALSE );
	
    hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_CURRENT);
    hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG2 );	


	//Here's details on what gets done below
	// - save off info about the viewport
	// - set target to above water
	//		- render above water background to above water scene
	//		- render objects above water
	//
	// - set target to below water
	//		- render below water background to below water scene
	//		- render objects below water
	//
	// - set target to environment map, using previous two scenes as textures
	//		- render above water scene onto environment map - upside down
	//      - render below water scene onto environment map - blended (to mimick transition
	//		  in quanitity of reflection)
	
	// RENDER THE FINAL SCENE INTO THE FRAME BUFFER, USING THE LAST SCENES AS TEXTURES
	// - set target to frame buffer
	//		- render above water scene to top 1/3
	//      - render triangle on bottom 2/3 using env map as texture (and bump map)
	

	// Here's the code for the above (no one said it would be pretty.
	
	// 0.0 - SAVE OFF THE OLD VIEWPORT
	hr = m_pWrapper->m_pD3DDevice->GetCurrentViewport(&m_D3DViewportMainApp);
	m_D3DViewportMainApp->GetViewport2(&m_D3DViewport2MainAppStruct);

	// 1 - RENDER WATER-REFLECTED SCENE ABOVE WATER ONTO A TEXTURE

	//		1.0 begin end scene pair. It's EndScene first, as our main.cpp has started a
	//		beginscene already.
	hr = m_pWrapper->m_pD3DDevice->EndScene();

	//		1.1 - set target to above water

	LPDIRECTDRAWSURFACE4 pdds4_above;
	hr = m_pAboveWaterTexture->ptexTexture->QueryInterface(IID_IDirectDrawSurface4,(void **)&pdds4_above);
	hr = m_pWrapper->m_pD3DDevice->SetRenderTarget(pdds4_above,0);
	if (hr != D3D_OK)
		OutputDebugString ("Looks like render to texture might not work on this HW \n");
	
	hr = m_D3DViewportAboveWater->SetViewport2(&m_D3DViewport2AboveWaterStruct);
	hr = m_pWrapper->m_pD3DDevice->SetCurrentViewport(m_D3DViewportAboveWater);

	if (m_bDrawWireframe)
		m_pWrapper->m_pD3DDevice->SetRenderState(D3DRENDERSTATE_FILLMODE,D3DFILL_WIREFRAME);

	hr = m_pWrapper->m_pD3DDevice->BeginScene();
	
	//		1.2 - clear
	D3DRECT rcClear;
	rcClear.x1 = 0; rcClear.y1 = 0;
	rcClear.x2 = texturesize; rcClear.y2 = texturesize;
	hr = m_D3DViewportAboveWater->Clear2( 1, &rcClear, D3DCLEAR_TARGET ,
										  0xff000000, 0.0f, 0 );

	//		1.3 - render above water background to above water scene
	hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, m_pAboveWaterTextureBackground->ptexTexture);	
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );	
	hr = m_pWrapper->m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX,
				m_rectAboveWaterBackgnd.verts, 6, D3DDP_DONOTLIGHT| D3DDP_DONOTCLIP);
	
	//		1.4 - render objects above water 
	//		(bird sprite)
	if (m_bDrawFish) 
	{
		hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, m_pBirdTexture->ptexTexture);	
		hr = m_pWrapper->m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX,
				m_rectSampleBird.verts, 6, D3DDP_DONOTLIGHT| D3DDP_DONOTCLIP);
	}

	//		(Gouraud shaded cube)
	if (m_bDrawCube) 
	{	hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, NULL);	
		hr = m_pWrapper->m_pD3DDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, D3DFVF_LVERTEX,
			m_SampleCubeVerts, NUMSAMPLECUBEVERTS, m_SampleCubeIndices,36, 0);
	}
	
	//		(GDI text on surface if requested
	HDC hdcSurface; 
	if (m_bDrawText)
	{
		if( DD_OK == pdds4_above->GetDC( &hdcSurface )  )
		{		
			TextOut(hdcSurface,70,300,"Party above water!",18);
			pdds4_above->ReleaseDC( hdcSurface );
		}		
	}

	//		1.5 Finish begin/endscene pair
	hr = m_pWrapper->m_pD3DDevice->EndScene();


	// That was the first of our 'scenes'. We will now repear for rendering the other scenes

	// 2 - RENDER SCENE BELOW WATER ONTO A TEXTURE
	//		2.1 - set target to above water
	LPDIRECTDRAWSURFACE4 pdds4_below;
	hr = m_pBelowWaterTexture->ptexTexture->QueryInterface(IID_IDirectDrawSurface4,(void **)&pdds4_below);
	hr = m_pWrapper->m_pD3DDevice->SetRenderTarget(pdds4_below,0);
	hr = m_D3DViewportBelowWater->SetViewport2(&m_D3DViewport2BelowWaterStruct);
	hr = m_pWrapper->m_pD3DDevice->SetCurrentViewport(m_D3DViewportBelowWater);

	hr = m_pWrapper->m_pD3DDevice->BeginScene();

	//		2.2 - clear
	rcClear.x1 = 0; rcClear.y1 = 0;
	rcClear.x2 = texturesize; rcClear.y2 = texturesize;
	hr = m_D3DViewportBelowWater->Clear2( 1, &rcClear, D3DCLEAR_TARGET , 0xff000000, 0.0f, 0 );

	//		2.3 - render below water background to below water scene
	hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, m_pBelowWaterTextureBackground->ptexTexture);	
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );	
	
	hr = m_pWrapper->m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX,
				m_rectBelowWaterBackgnd.verts, 6, D3DDP_DONOTLIGHT| D3DDP_DONOTCLIP);
	
	//		2.4 - render objects below water	
	//		(fish sprite)
	if (m_bDrawFish)
	{
		hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, m_pFishTexture->ptexTexture);		
		hr = m_pWrapper->m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX,
				m_rectSampleFish.verts, 6, D3DDP_DONOTLIGHT| D3DDP_DONOTCLIP);
	}
	
	//		(GDI Text if requested)
	if (m_bDrawText)
	{
		if( DD_OK == pdds4_below->GetDC( &hdcSurface )  )
		{		
			TextOut(hdcSurface,70,300,"Party under water!",18);
			pdds4_below->ReleaseDC( hdcSurface );
		}
	}
	
	//		2.5 Finish begin/endscene pair
	hr = m_pWrapper->m_pD3DDevice->EndScene();

	
	// 3 - RENDER 2 RECTS ONTO ENVMAPTEXTURE, ONE WITH UNDERWATERSCENE, 
	//     ONE WITH ABOVEWATERSCENE FLIPPED UPSIDE DOWN, BLEND THEM TOGETHER

	//		3.1 - set target to above water
	LPDIRECTDRAWSURFACE4 pdds4_Env;
	hr = m_pEnvMapTexture->ptexTexture->QueryInterface(IID_IDirectDrawSurface4,(void **)&pdds4_Env);
	hr = m_pWrapper->m_pD3DDevice->SetRenderTarget(pdds4_Env,0);
	hr = m_D3DViewportEnvMap->SetViewport2(&m_D3DViewport2EnvMapStruct);
	hr = m_pWrapper->m_pD3DDevice->SetCurrentViewport(m_D3DViewportEnvMap);
	
	hr = m_pWrapper->m_pD3DDevice->BeginScene();
	
	// If the wireframe view was selected earlier, then we need to disable it now. I want to actually
	// render a scene which uses the previous scenes 'framebuffers' in wireframe as textures
	if (m_bDrawWireframe)
		m_pWrapper->m_pD3DDevice->SetRenderState(D3DRENDERSTATE_FILLMODE,D3DFILL_SOLID);
	
	//		3.2 - clear
	rcClear.x1 = 0; rcClear.y1 = 0;
	rcClear.x2 = texturesize; rcClear.y2 = texturesize;
	hr = m_D3DViewportEnvMap->Clear2( 1, &rcClear, D3DCLEAR_TARGET , 0xff000000, 0.0f, 0 );

	//		3.3 - render 2 rectangles onto envmap, one with underwater scene texture created in step 1, 
	//            one with above waterscene texture created in step 2
	hr = m_pWrapper->m_pD3DDevice->SetRenderState( D3DRENDERSTATE_TEXTUREMAPBLEND , D3DTBLEND_MODULATE);
	hr = m_pWrapper->m_pD3DDevice->SetRenderState( D3DRENDERSTATE_SRCBLEND, D3DBLEND_ONE);
	hr = m_pWrapper->m_pD3DDevice->SetRenderState( D3DRENDERSTATE_DESTBLEND, D3DBLEND_ONE);
		

	hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, m_pBelowWaterTexture->ptexTexture);	
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );	//SELECTARG1 );	
	
	hr = m_pWrapper->m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX,
	m_rectBelowWaterScene.verts, 6, D3DDP_DONOTLIGHT| D3DDP_DONOTCLIP);
	
	hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, m_pAboveWaterTexture->ptexTexture);	
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );	
	hr = m_pWrapper->m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX,
				m_rectAboveWaterScene.verts, 6, D3DDP_DONOTLIGHT| D3DDP_DONOTCLIP);
	
	//	Draw GDI Text on the surface if requested	
	if (m_bDrawText)
	{
		if( DD_OK == pdds4_Env->GetDC( &hdcSurface )  )
		{		
			TextOut(hdcSurface,30,350,"Party on the envmap!",20);
			pdds4_Env->ReleaseDC( hdcSurface );
		}
	}

	//		3.4 Finish begin/endscene pair
	hr = m_pWrapper->m_pD3DDevice->EndScene();

	// 4. RENDER THE FINAL SCENE, BY (A) RENDERING THE SCENE ABOVE WATER USING A RECTANGLE AND THE ABOVETEXTURE
	//                               (B) RENDERING THE WATER USING A RECTANGLE, A BUMP MAP, AND THE ENVMAP
	//
	//		4.1 Set the render target back to the back buffer, and reset the viewport
	hr = m_pWrapper->m_pD3DDevice->SetCurrentViewport(m_D3DViewportMainApp);
	hr = m_pWrapper->m_pD3DDevice->SetRenderTarget(m_pWrapper->m_pBackBuffer,0);
	m_D3DViewportMainApp->SetViewport2(&m_D3DViewport2MainAppStruct);
	
	hr = m_pWrapper->m_pD3DDevice->BeginScene();

	//		4.3 render tris above water - for now this is a copy of the reflected scene, but
	//      should be poly's from a different viewpoint.
	hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, m_pAboveWaterTexture->ptexTexture);	
	hr = m_pWrapper->m_pD3DDevice->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, TRUE);
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);
	hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );	
	hr = m_pWrapper->m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX,
				m_rectFinalAbove.verts, 6, D3DDP_DONOTLIGHT);

	//	Render water tri's, but need to handle different flags (if bump mapping or not, etc)
	if((m_bDrawBumps == false) || (m_bDeviceDoesBumpMapping == false))
	{
	//		4.4 render water tris with envmap texture
		hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, m_pEnvMapTexture->ptexTexture);	
		hr = m_pWrapper->m_pD3DDevice->SetTexture( 1, NULL);	
		if (m_bDrawWater) 
		{
			m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, 0 );
			hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
			hr = m_pWrapper->m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);
			hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );	
		
			hr = m_pWrapper->m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 
				D3DFVF_XYZRHW|D3DFVF_DIFFUSE|D3DFVF_TEX2,
				m_rectFinalBelow.verts, /*3*/6, D3DDP_DONOTLIGHT);
		}
	}
	else
	{
		// Render water texture with waves using bump mapping
		// stage 0 points to the bump map, stage 1 to the water scene environment map
		if (m_bProcedural)
			hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, m_pBumpMapProcTexture->ptexTexture );
		else
			hr = m_pWrapper->m_pD3DDevice->SetTexture( 0, m_pBumpMapAnimTexture->ptexTexture );

		hr = m_pWrapper->m_pD3DDevice->SetTexture( 1, m_pEnvMapTexture->ptexTexture );

		//set all the bump mapping render states (and others we'll need)
		
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, 0 );
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_BUMPENVMAP );
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_CURRENT );
		// the 00 and 11 elements of the bump mapping matrix dictate the amount of peturbation. I chose to 
		// use small values for a very subtle effect. Anything too large looks ugly (for this case)
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_BUMPENVMAT00,   FLOATtoDWORD(0.03f) );
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_BUMPENVMAT01,   FLOATtoDWORD(0.00f) );
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_BUMPENVMAT10,   FLOATtoDWORD(0.00f) );
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_BUMPENVMAT11,   FLOATtoDWORD(0.03f) );
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_BUMPENVLSCALE,  FLOATtoDWORD(1.00f) );
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_BUMPENVLOFFSET, FLOATtoDWORD(0.00f) );

		//set the states for the envmap (stage 1)
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 1, D3DTSS_TEXCOORDINDEX, 1);
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLORARG1, D3DTA_TEXTURE );
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLORARG2, D3DTA_CURRENT );
		
		//		4.5 render water tris with envmap texture	
		if (m_bDrawWater) 
		{
			hr = m_pWrapper->m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_XYZRHW|D3DFVF_DIFFUSE|D3DFVF_TEX2,
				m_rectFinalBelow.verts, /*3*/6, D3DDP_DONOTLIGHT);
		}

		//disable these funky states, as main.cpp called this, and may want to draw something else next
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_DISABLE );
		hr = m_pWrapper->m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLOROP,   D3DTOP_DISABLE );

	}

	return S_OK;
}


// -----------------------------------------------------------------------
// HRESULT CWater::TargetChanged( void )
//
// This indicates that the devices of the DRGWrapper class have changed.
// If you have anything that depends on the current rendering device, you'll
// have to re-initialize it here
// -----------------------------------------------------------------------
HRESULT CWater::TargetChanged( void )
{
	HRESULT hr;
	//
	//
	if (m_pWrapper->m_bHardware)
	{
		if (m_pWrapper->m_pCurrentDriver->HALDesc.dwTextureOpCaps & D3DTEXOPCAPS_BUMPENVMAP)
			m_bDeviceDoesBumpMapping = true;
		else
			m_bDeviceDoesBumpMapping = false;
	}
	else
	{
		if (m_pWrapper->m_pCurrentDriver->HELDesc.dwTextureOpCaps & D3DTEXOPCAPS_BUMPENVMAP)
			m_bDeviceDoesBumpMapping = true;
		else
			m_bDeviceDoesBumpMapping = false;
	}

	m_pWrapper->m_pD3DDevice->SetRenderState( D3DRENDERSTATE_DITHERENABLE, TRUE );
	
	m_pWrapper->m_pD3DDevice->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
	m_pWrapper->m_pD3DDevice->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );
	m_pWrapper->m_pD3DDevice->SetTransform( D3DTRANSFORMSTATE_PROJECTION, &m_MxProjection.m_Mx );

	//restore textures
	if (m_pEnvMapTexture)
		m_pEnvMapTexture->Restore(m_pWrapper->m_pD3DDevice);
	if (m_pAboveWaterTexture)
		m_pAboveWaterTexture->Restore(m_pWrapper->m_pD3DDevice);
	if (m_pBelowWaterTexture)
		m_pBelowWaterTexture->Restore(m_pWrapper->m_pD3DDevice);

	if (m_pEnvMapTextureBackground)
		m_pEnvMapTextureBackground->Restore(m_pWrapper->m_pD3DDevice);
	if (m_pAboveWaterTextureBackground)
		m_pAboveWaterTextureBackground->Restore(m_pWrapper->m_pD3DDevice);
	if (m_pBelowWaterTextureBackground)
		m_pBelowWaterTextureBackground->Restore(m_pWrapper->m_pD3DDevice);

	if (m_pFishTexture)
		m_pFishTexture->Restore(m_pWrapper->m_pD3DDevice);
	if (m_pBirdTexture)
		m_pBirdTexture->Restore(m_pWrapper->m_pD3DDevice);

	if (m_bDeviceDoesBumpMapping)
	{
		if (m_pBumpMapProcTexture)
			m_pBumpMapProcTexture->Restore(m_pWrapper->m_pD3DDevice);

		if (m_pBumpMapAnimTexture)
			m_pBumpMapAnimTexture->Restore(m_pWrapper->m_pD3DDevice);
	}


	//re-attach viewport
	hr = m_pWrapper->m_pD3DDevice->AddViewport(m_D3DViewportEnvMap);
	hr = m_pWrapper->m_pD3DDevice->AddViewport(m_D3DViewportAboveWater);
	hr = m_pWrapper->m_pD3DDevice->AddViewport(m_D3DViewportBelowWater);

	// - set up a new viewport2 to be used for the next few scenes
	m_D3DViewport2EnvMapStruct.dwSize = sizeof(D3DVIEWPORT);
	m_D3DViewport2EnvMapStruct.dwX =			0; 
	m_D3DViewport2EnvMapStruct.dwY =			0;
	m_D3DViewport2EnvMapStruct.dwWidth =		texturesize;  //hardcoded for now, but should change
	m_D3DViewport2EnvMapStruct.dwHeight =		texturesize;  // by adding a size member var to the texture class
	m_D3DViewport2EnvMapStruct.dvClipX =		-1.0f;
	m_D3DViewport2EnvMapStruct.dvClipY =		1.0f;
	m_D3DViewport2EnvMapStruct.dvClipWidth =	2.0f;
	m_D3DViewport2EnvMapStruct.dvClipHeight = 2.0f;
	m_D3DViewport2EnvMapStruct.dvMinZ =		0.0f;
	m_D3DViewport2EnvMapStruct.dvMaxZ =		1.0f;

		// - set up a new viewport2 to be used for the next few scenes
	m_D3DViewport2BelowWaterStruct.dwSize = sizeof(D3DVIEWPORT);
	m_D3DViewport2BelowWaterStruct.dwX =			0; 
	m_D3DViewport2BelowWaterStruct.dwY =			0;
	m_D3DViewport2BelowWaterStruct.dwWidth =		texturesize;  //hardcoded for now, but should change
	m_D3DViewport2BelowWaterStruct.dwHeight =		texturesize;  // by adding a size member var to the texture class
	m_D3DViewport2BelowWaterStruct.dvClipX =		-1.0f;
	m_D3DViewport2BelowWaterStruct.dvClipY =		1.0f;
	m_D3DViewport2BelowWaterStruct.dvClipWidth =	2.0f;
	m_D3DViewport2BelowWaterStruct.dvClipHeight = 2.0f;
	m_D3DViewport2BelowWaterStruct.dvMinZ =		0.0f;
	m_D3DViewport2BelowWaterStruct.dvMaxZ =		1.0f;

		// - set up a new viewport2 to be used for the next few scenes
	m_D3DViewport2AboveWaterStruct.dwSize = sizeof(D3DVIEWPORT);
	m_D3DViewport2AboveWaterStruct.dwX =			0; 
	m_D3DViewport2AboveWaterStruct.dwY =			0;
	m_D3DViewport2AboveWaterStruct.dwWidth =		texturesize;  //hardcoded for now, but should change
	m_D3DViewport2AboveWaterStruct.dwHeight =		texturesize;  // by adding a size member var to the texture class
	m_D3DViewport2AboveWaterStruct.dvClipX =		-1.0f;
	m_D3DViewport2AboveWaterStruct.dvClipY =		1.0f;
	m_D3DViewport2AboveWaterStruct.dvClipWidth =	2.0f;
	m_D3DViewport2AboveWaterStruct.dvClipHeight = 2.0f;
	m_D3DViewport2AboveWaterStruct.dvMinZ =		0.0f;
	m_D3DViewport2AboveWaterStruct.dvMaxZ =		1.0f;

	return S_OK;
}


// -----------------------------------------------------------------------
// void CWater::SetupFloowVerts(float scale)
//
//   this is just a function that sets up 2 tris to draw a cube. The only reason it
//   it is pulled out into a separate function is the make the init code easier to read
// -----------------------------------------------------------------------
void CWater::SetupSampleCubeVerts(float scale)
{

	memset( m_SampleCubeVerts, 0, sizeof( D3DLVERTEX ) * NUMSAMPLECUBEVERTS );

	//this will essentially be two triangles located at Y=-1.0f for now
	m_SampleCubeVerts[0].x = -scale;	m_SampleCubeVerts[0].y = -scale;	m_SampleCubeVerts[0].z = -scale;
	m_SampleCubeVerts[1].x = -scale;	m_SampleCubeVerts[1].y = -scale;	m_SampleCubeVerts[1].z = scale;
	m_SampleCubeVerts[2].x = -scale;	m_SampleCubeVerts[2].y =  scale;	m_SampleCubeVerts[2].z = -scale;
	m_SampleCubeVerts[3].x = -scale;	m_SampleCubeVerts[3].y =  scale;	m_SampleCubeVerts[3].z = scale;
	m_SampleCubeVerts[4].x =  scale;	m_SampleCubeVerts[4].y = -scale;	m_SampleCubeVerts[4].z = -scale;
	m_SampleCubeVerts[5].x =  scale;	m_SampleCubeVerts[5].y = -scale;	m_SampleCubeVerts[5].z = scale;
	m_SampleCubeVerts[6].x =  scale;	m_SampleCubeVerts[6].y =  scale;	m_SampleCubeVerts[6].z = -scale;
	m_SampleCubeVerts[7].x =  scale;	m_SampleCubeVerts[7].y =  scale;	m_SampleCubeVerts[7].z = scale;
	//face 1
	m_SampleCubeIndices[0] = 1;m_SampleCubeIndices[1] = 2;m_SampleCubeIndices[2] = 0;
	m_SampleCubeIndices[3] = 1;m_SampleCubeIndices[4] = 3;m_SampleCubeIndices[5] = 2;
	//face 2
	m_SampleCubeIndices[6] = 5;m_SampleCubeIndices[7] = 3;m_SampleCubeIndices[8] = 1;
	m_SampleCubeIndices[9] = 5;m_SampleCubeIndices[10] = 7;m_SampleCubeIndices[11] = 3;
	//face 3
	m_SampleCubeIndices[12] = 4;m_SampleCubeIndices[13] = 7;m_SampleCubeIndices[14] = 5;
	m_SampleCubeIndices[15] = 4;m_SampleCubeIndices[16] = 6;m_SampleCubeIndices[17] = 7;
	//face 4
	m_SampleCubeIndices[18] = 0;m_SampleCubeIndices[19] = 6;m_SampleCubeIndices[20] = 4;
	m_SampleCubeIndices[21] = 0;m_SampleCubeIndices[22] = 2;m_SampleCubeIndices[23] = 6;
	//face 5
	m_SampleCubeIndices[24] = 7;m_SampleCubeIndices[25] = 2;m_SampleCubeIndices[26] = 3;
	m_SampleCubeIndices[27] = 7;m_SampleCubeIndices[28] = 6;m_SampleCubeIndices[29] = 2;
	//face 6
	m_SampleCubeIndices[30] = 4;m_SampleCubeIndices[31] = 1;m_SampleCubeIndices[32] = 0;
	m_SampleCubeIndices[33] = 4;m_SampleCubeIndices[34] = 5;m_SampleCubeIndices[35] = 1;
	

	m_SampleCubeVerts[0].tu = 0.0f; m_SampleCubeVerts[0].tv = 1.0f;
	m_SampleCubeVerts[1].tu = 1.0f; m_SampleCubeVerts[1].tv = 1.0f;
	m_SampleCubeVerts[2].tu = 1.0f; m_SampleCubeVerts[2].tv = 0.0f;
	m_SampleCubeVerts[3].tu = 0.0f; m_SampleCubeVerts[3].tv = 1.0f;
	m_SampleCubeVerts[4].tu = 1.0f; m_SampleCubeVerts[4].tv = 0.0f;
	m_SampleCubeVerts[5].tu = 0.0f; m_SampleCubeVerts[5].tv = 0.0f;
	m_SampleCubeVerts[6].tu = 0.0f; m_SampleCubeVerts[5].tv = 0.0f;
	m_SampleCubeVerts[7].tu = 0.0f; m_SampleCubeVerts[5].tv = 0.0f;
	m_SampleCubeVerts[5].tu = 0.0f; m_SampleCubeVerts[5].tv = 0.0f;
	m_SampleCubeVerts[5].tu = 0.0f; m_SampleCubeVerts[5].tv = 0.0f;
	
	// common element
	for (int i=0;i<NUMSAMPLECUBEVERTS;i++)
		m_SampleCubeVerts[i].dcSpecular = 0;

	m_SampleCubeVerts[0].dcColor = D3DRGB( 1.0f, 0.0f, 0.0f );
	m_SampleCubeVerts[1].dcColor = D3DRGB( 0.0f, 1.0f, 0.0f );
	m_SampleCubeVerts[2].dcColor = D3DRGB( 0.0f, 0.0f, 1.0f );
	m_SampleCubeVerts[3].dcColor = D3DRGB( 1.0f, 1.0f, 0.0f );
	m_SampleCubeVerts[4].dcColor = D3DRGB( 1.0f, 0.0f, 1.0f );
	m_SampleCubeVerts[5].dcColor = D3DRGB( 0.0f, 1.0f, 1.0f );
	m_SampleCubeVerts[6].dcColor = D3DRGB( 0.0f, 0.0f, 0.0f );
	m_SampleCubeVerts[7].dcColor = D3DRGB( 1.0f, 1.0f, 1.0f );
	
	return;
}


//-----------------------------------------------------------------------------
// Name: InitBumpMapArray()
// Desc: sets up bits in an array to be used for our one-time generated texture
//-----------------------------------------------------------------------------
void CWater::InitBumpMapArray(int Hres, int Vres, char *pArray)
{
	char *pDst = pArray;
	for( DWORD y=0; y<(DWORD)Vres; y++ )
	{
		for( DWORD x=0; x<(DWORD)(2*Hres); x++ )
		{
			FLOAT fx = x/(FLOAT)Hres - 0.5f;
			FLOAT fy = y/(FLOAT)Vres - 0.5f;

			FLOAT r = sqrtf( fx*fx + fy*fy );
	
			char cDu = (char)(32*cos(4.0*(fx + fy)*3.14f*4.0f));
			char cDv = (char)(32*sin(4.0*(fx + fy)*3.14f*4.0f));
			        
			// In the U8V8 format, 1st byte is U, 2nd byte is V.
			pDst[2*x+0] = cDu;
			pDst[2*x+1] = cDv;
		}
		pDst += 4*Hres;
	}	
	return;
}



// -----------------------------------------------------------------------
// void CWater::KeyUp(WPARAM wParam)
//
//   Keyboard keypress handler. 
// -----------------------------------------------------------------------
void CWater::KeyDown(WPARAM wParam)
{
	//we need a variable to modify our rotation speed with frame rate	
	float scaler = 1.0f;

	//if exiting, the wrapper could be invalid, but we don't care, since we will exit anyway
	if (wParam == 'X') 
		return;

	if(m_pWrapper) //we could get into this loop after the wrapper has been invalidates
	{
		if (m_pWrapper->m_fFrameTime >0.0f)
			scaler = m_pWrapper->m_fFrameTime;  
	}
		
	switch (wParam) 
	{
	case VK_UP:
		m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, -3.14159f*scaler );
		break;
	
	case VK_DOWN:
		m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, 3.14159f*scaler );
		break;
	
	case VK_LEFT:
		m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, -3.14159f*scaler );
		break;
	
	case VK_RIGHT:	
		m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, 3.14159f*scaler );
		break;

	case 'C':
		m_bDrawCube = !m_bDrawCube;		
		break;

	case 'A':
		m_bDrawWater = !m_bDrawWater;
		break;

	case 'B':		
		m_bDrawBumps = !m_bDrawBumps;
		if (!m_bDeviceDoesBumpMapping)
			m_bDrawBumps = false;
		break;

	case 'U':
		m_bProcedural = !m_bProcedural;
		//we may need to reset these, as they may have been animated
		if (m_bProcedural)
		{
			m_rectFinalBelow.verts[0].tu = 0.05f;
			m_rectFinalBelow.verts[3].tu = 0.05f;
			m_rectFinalBelow.verts[5].tu = 0.05f;

			m_rectFinalBelow.verts[1].tu = 0.95f;
			m_rectFinalBelow.verts[2].tu = 0.95f;
			m_rectFinalBelow.verts[4].tu = 0.95f;

			m_rectFinalBelow.verts[0].tv = 0.05f;
			m_rectFinalBelow.verts[1].tv = 0.05f;
			m_rectFinalBelow.verts[3].tv = 0.05f;

			m_rectFinalBelow.verts[2].tv = 0.95f;
			m_rectFinalBelow.verts[4].tv = 0.95f;
			m_rectFinalBelow.verts[5].tv = 0.95f;
		}
		break;

	case 'V':
		m_bProceduralWaves = !m_bProceduralWaves;
		break;
	case 'R':
		m_bProceduralRain = !m_bProceduralRain;		
		break;

	case 'I':
		m_bDrawFish = !m_bDrawFish;	
		break;

	case 'T':
		m_bDrawText = !m_bDrawText;
		break;

	case 'P':
		m_bPauseAnim = !m_bPauseAnim;
		break;

	case 'W':
		m_bDrawWireframe = !m_bDrawWireframe;
		break;

	}

	if (m_pWrapper)
	{
		m_pWrapper->m_pD3DDevice->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
		m_pWrapper->m_pD3DDevice->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );
	}

	
	return;
}

void CWater::KeyUp(WPARAM wParam)
{		
	switch (wParam) 
	{
		//add code here to handle key releases, if you need them
	case VK_UP:
		OutputDebugString("The up arrow was released;");			
		break;
	}
	
	return;
}


void CWater::MouseLeftUp(WPARAM wParam, LPARAM lParam)
{	
	return;
}

void CWater::MouseLeftDown(WPARAM wParam, LPARAM lParam)
{		
	int x = LOWORD(lParam);
	int y = HIWORD(lParam);
		
	if (m_bProcedural)
	{
		if (y > (0.33f*m_pWrapper->m_dwRenderHeight))
		{
			
			int iTexX = (x*PROCTEXSIZE)/m_pWrapper->m_dwRenderWidth;
	
			int iTexY = (y-(0.33f*m_pWrapper->m_dwRenderHeight)) * 1.5f * PROCTEXSIZE / m_pWrapper->m_dwRenderHeight;

			iTexX = iTexX * 0.9f + (PROCTEXSIZE * 0.05f);
			iTexY = iTexY * 0.9f + (PROCTEXSIZE * 0.05f);
			
			m_ClickBuffer[iTexY*PROCTEXSIZE+iTexX] = 22;

		}
	}

	return;
}

void CWater::MouseRightUp(WPARAM wParam, LPARAM lParam)
{	
	return;
}

void CWater::MouseRightDown(WPARAM wParam, LPARAM lParam)
{		
	return;
}

void CWater::MouseMove(WPARAM wParam, LPARAM lParam)
{
	if ( wParam & MK_LBUTTON )	// Left button down plus mouse move == rotate
	{
		//we need a variable to modify our rotation speed with frame rate	
		float scaler = 1.0f;
		if(m_pWrapper) //we could get into this loop after the wrapper has been invalidates
		{
			scaler = 2.0f / max(m_pWrapper->m_dwRenderWidth, m_pWrapper->m_dwRenderHeight);  
		}

		//we need to compare the last position to the current position
		int x = LOWORD(lParam);
		int y = HIWORD(lParam);
		
		if (x - m_iMousePrevX)
		{
			m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, 3.14159f*scaler*(float)(x - m_iMousePrevX) );
		}
			
		if (y - m_iMousePrevY)
		{
			m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, 3.14159f*scaler*(float)(y - m_iMousePrevY) );
		}

		m_pWrapper->m_pD3DDevice->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
		m_pWrapper->m_pD3DDevice->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );

	
		m_iMousePrevX = x;
		m_iMousePrevY = y;

		if (m_bProcedural)
		{
			if (y > (0.33f*m_pWrapper->m_dwRenderHeight))
			{
				
				int iTexX = (x*PROCTEXSIZE)/m_pWrapper->m_dwRenderWidth;
		
				int iTexY = (y-(0.33f*m_pWrapper->m_dwRenderHeight)) * 1.5f * PROCTEXSIZE / m_pWrapper->m_dwRenderHeight;

				iTexX = iTexX * 0.9f + (PROCTEXSIZE * 0.05f);
				iTexY = iTexY * 0.9f + (PROCTEXSIZE * 0.05f);
				
				m_ClickBuffer[iTexY*PROCTEXSIZE+iTexX] = 22;

			}
		}

	}
	else if ( wParam & MK_RBUTTON )	// Right button down plus mouse move == Zoom camera in/out
	{
		//we need a variable to modify our rotation speed with frame rate	
		float scaler = 1.0f;
		if(m_pWrapper) //we could get into this loop after the wrapper has been invalidates
		{
			scaler = 5.0f / max(m_pWrapper->m_dwRenderWidth, m_pWrapper->m_dwRenderHeight);  
		}

		//we need to compare the last position to the current position
		int x = LOWORD(lParam);
		int y = HIWORD(lParam);
		
		if (y - m_iMousePrevY)
		{
			DRGMatrix Temp(true);

			Temp.Translate( 0,0,scaler * (y - m_iMousePrevY) );
			m_MxView.PostMultiply( Temp );
		}

		m_pWrapper->m_pD3DDevice->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
		m_pWrapper->m_pD3DDevice->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );

	
		m_iMousePrevX = x;
		m_iMousePrevY = y;

	}
	else
	{
		//need to update these, or the next time someone clicks, the delta is massive
		m_iMousePrevX = LOWORD(lParam);
		m_iMousePrevY = HIWORD(lParam);	
	}
	return;
}

