// Water.h
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
// Water.h: declaration of the Water class.
// This class is the "blank" class created by the DRG 3D Appwizard, where you will put your stuff
//
// ------------------------------------------------------------------------------------
//
// Author: Kim Pallister - Intel Developer Relations & Engineering Group
//
////////////////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_DRGAPP_H__CFF24425_47D6_11D2_B1A5_00A0C90D2C1B__INCLUDED_)
#define AFX_DRGAPP_H__CFF24425_47D6_11D2_B1A5_00A0C90D2C1B__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


//this is a vertex format struct so I can have multiple UV's per vert
struct DualTexVertexFormat
{
   float sx;
   float sy;
   float sz;
   float rhw;
   DWORD color;
   float tu;
   float tv;
   float tu2;
   float tv2;
};

//a struct for a rectangle we'll use for all these square regions on screen
struct ScreenRectangle {

	D3DTLVERTEX	verts[6];

	ScreenRectangle::ScreenRectangle()
	{
		for(int i = 0;i<6;i++)
		{
			verts[i].sx = 0.0f;
			verts[i].sy = 0.0f;
			verts[i].sz = 0.99f;
			verts[i].rhw = 1.0f;
			verts[i].color = 0xffffffff;
		}

		//for test purposes only get rid of this once we start texturing
		
		verts[0].color = 0xff2222ff; //red
		verts[1].color = 0xff22ff22; //green
		verts[2].color = 0xffff2222; //blue
		verts[3].color = 0xffffff22; //yellow
		verts[4].color = 0xffff22ff; //purple
		verts[5].color = 0xff22ffff; //cyan
		
				verts[0].tu = 0.0f; verts[0].tv = 0.0f;
		verts[1].tu = 1.0f; verts[1].tv = 0.0f;
		verts[2].tu = 1.0f; verts[2].tv = 1.0f;
		verts[3].tu = 0.0f; verts[3].tv = 0.0f;
		verts[4].tu = 1.0f; verts[4].tv = 1.0f;
		verts[5].tu = 0.0f; verts[5].tv = 1.0f;
	}

	void SetSize (float x1,float y1,float x2,float y2)
	{
		verts[0].sx = x1; verts[0].sy = y1;
		verts[1].sx = x2; verts[1].sy = y1;
		verts[2].sx = x2; verts[2].sy = y2;
		verts[3].sx = x1; verts[3].sy = y1;
		verts[4].sx = x2; verts[4].sy = y2;
		verts[5].sx = x1; verts[5].sy = y2;
		return;
	}
	void SetZ (float z)
	{
		verts[0].sz = z; 
		verts[1].sz = z;
		verts[2].sz = z;
		verts[3].sz = z;
		verts[4].sz = z;
		verts[5].sz = z;
		return;
	}
};

//a struct for a rectangle we'll use for all these square regions on screen
struct ScreenRectangleDual {

	DualTexVertexFormat	verts[6];
	
	ScreenRectangleDual::ScreenRectangleDual()
	{
		for(int i = 0;i<6;i++)
		{
			verts[i].sx = 0.0f;
			verts[i].sy = 0.0f;
			verts[i].sz = 0.99f;
			verts[i].rhw = 1.0f;
			verts[i].color = 0xffffffff;
		}

		//for test purposes only get rid of this once we start texturing
		verts[0].color = 0xff2222ff; //red
		verts[1].color = 0xff22ff22; //green
		verts[2].color = 0xffff2222; //blue
		verts[3].color = 0xffffff22; //yellow
		verts[4].color = 0xffff22ff; //purple
		verts[5].color = 0xff22ffff; //cyan

		verts[0].tu = 0.0f; verts[0].tv = 0.0f;
		verts[1].tu = 1.0f; verts[1].tv = 0.0f;
		verts[2].tu = 1.0f; verts[2].tv = 1.0f;
		verts[3].tu = 0.0f; verts[3].tv = 0.0f;
		verts[4].tu = 1.0f; verts[4].tv = 1.0f;
		verts[5].tu = 0.0f; verts[5].tv = 1.0f;

		verts[0].tu2 = 0.0f; verts[0].tv2 = 0.0f;
		verts[1].tu2 = 1.0f; verts[1].tv2 = 0.0f;
		verts[2].tu2 = 1.0f; verts[2].tv2 = 1.0f;
		verts[3].tu2 = 0.0f; verts[3].tv2 = 0.0f;
		verts[4].tu2 = 1.0f; verts[4].tv2 = 1.0f;
		verts[5].tu2 = 0.0f; verts[5].tv2 = 1.0f;
	}

	void SetSize (float x1,float y1,float x2,float y2)
	{
		verts[0].sx = x1; verts[0].sy = y1;
		verts[1].sx = x2; verts[1].sy = y1;
		verts[2].sx = x2; verts[2].sy = y2;
		verts[3].sx = x1; verts[3].sy = y1;
		verts[4].sx = x2; verts[4].sy = y2;
		verts[5].sx = x1; verts[5].sy = y2;
		return;
	}
	void SetZ (float z)
	{
		verts[0].sz = z; 
		verts[1].sz = z;
		verts[2].sz = z;
		verts[3].sz = z;
		verts[4].sz = z;
		verts[5].sz = z;
		return;
	}
};


#define	NUMSAMPLEBACKGROUNDVERTS	6

#define	NUMSAMPLEAXISVERTS	22 

#define	NUMSAMPLECUBEVERTS	8
#define	NUMSAMPLEFISHVERTS	6

//I ran out of vidmem making this 256, but it can be made 64, 32
#define PROCTEXSIZE			128


class CWater  
{
    CDRGWrapper *m_pWrapper;
	

	int  m_iMousePrevX;
	int  m_iMousePrevY;

	//Bump mapping demo specific member vars
	////////////////////////////////////////

	D3DTLVERTEX m_SampleBackgroundVerts[NUMSAMPLEBACKGROUNDVERTS];	
	bool	m_bDrawBackground;

	D3DLVERTEX	m_SampleCubeVerts[NUMSAMPLECUBEVERTS];	
	WORD		m_SampleCubeIndices[36];	//6*6*2

	bool	m_bDeviceDoesBumpMapping;

	bool	m_bDrawCube;		//draw a polygonal cube
	bool	m_bDrawWater;		//draw the two poly's representing the water
	bool	m_bProcedural;		//toggle use of procedural texture vs UV animated
	bool	m_bProceduralWaves; // do a procedural wave effect on texture
	bool	m_bProceduralRain;	// do a procedural rain effect on texture
	bool	m_bDrawFish;		//draw fish and bird sprites
	bool	m_bDrawBumps;		//toggle DX6 bump mapping
	bool	m_bDrawText;		//toggle GDI text drawn on buffers
	bool	m_bDrawWireframe;	// draw wireframe on texture surfaces

	bool	m_bPauseAnim;	

	
	//these just get drawn as rects for now
	ScreenRectangle m_rectSampleFish;
	float   m_fFishPosX, m_fFishPosY; 
	ScreenRectangle m_rectSampleBird;
	float   m_fBirdPosX, m_fBirdPosY;
	
	
	int	m_iSinTimes32[256]; //lookup tables of 32 * sin [n]
	int	m_iCosTimes32[256];
	DWORD	m_dwFrameCount;
	
	int	m_ClickBuffer[PROCTEXSIZE*PROCTEXSIZE];   //stores where the mouse was clicked
	WORD	m_DripTexBuffer[PROCTEXSIZE*PROCTEXSIZE]; //generate waves from the mouse clicks
	WORD	m_ProcTexBuffer[PROCTEXSIZE*PROCTEXSIZE]; //here we will procedurally generate waves
	WORD	m_CompTexBuffer[PROCTEXSIZE*PROCTEXSIZE]; //to composite the two waves
	
	
	
	DRGVector	m_vCameraVector;

	//for view as seen from viewer
	DRGMatrix m_MxWorld;
	DRGMatrix m_MxProjection;
	DRGMatrix m_MxView;

	//for view underwater as seen by viewer, refracted through water 
	DRGMatrix m_MxRefractedWorld;		//these will be used to render the scene to the underwater texture
	DRGMatrix m_MxRefractedProjection;
	DRGMatrix m_MxRefractedView;
	//for view of world as reflected off water
	DRGMatrix m_MxReflectedWorld;
	DRGMatrix m_MxReflectedProjection;
	DRGMatrix m_MxReflectedView;

	float  m_fEnvMapSplit;
	float  m_fCompositeRefractPoint;
	float  m_fCompositeShoreline;

	LPDIRECT3DVIEWPORT3	m_D3DViewportMainApp;
	D3DVIEWPORT2	m_D3DViewport2MainAppStruct;
	
	LPDIRECT3DVIEWPORT3	m_D3DViewportBelowWater;
	D3DVIEWPORT2	m_D3DViewport2BelowWaterStruct;
	
	LPDIRECT3DVIEWPORT3	m_D3DViewportAboveWater;
	D3DVIEWPORT2	m_D3DViewport2AboveWaterStruct;
	
	LPDIRECT3DVIEWPORT3	m_D3DViewportEnvMap;
	D3DVIEWPORT2	m_D3DViewport2EnvMapStruct;

	//these rectangles are triangle pairs we 
	// will use when rendering onto textures
	ScreenRectangle	m_rectBelowWaterScene;
	ScreenRectangle	m_rectBelowWaterBackgnd;

	ScreenRectangle	m_rectAboveWaterScene;
	ScreenRectangle	m_rectAboveWaterBackgnd;
	
	ScreenRectangle m_rectEnvMapBelow;
	ScreenRectangle m_rectEnvMapAbove;

	ScreenRectangleDual m_rectFinalBelow;
	ScreenRectangle m_rectFinalAbove;

	//texture member vars for both the
	// texture and target textures
	CDRGTexture *m_pEnvMapTextureBackground;
	CDRGTexture *m_pAboveWaterTextureBackground;
	CDRGTexture *m_pBelowWaterTextureBackground;

	CDRGTexture *m_pEnvMapTexture;
	CDRGTexture *m_pAboveWaterTexture;
	CDRGTexture *m_pBelowWaterTexture;

	CDRGTexture *m_pFishTexture;
	CDRGTexture *m_pBirdTexture;

	CDRGTexture *m_pBumpMapProcTexture;
	CDRGTexture *m_pBumpMapAnimTexture;


	//Because our DRGTexture class doesn't
	// have bump mapping support yet, these
	// are created instead
//	LPDIRECT3DTEXTURE2 m_pBumpMapTexture;
//	LPDIRECTDRAWSURFACE4 m_pBumpMapSurface;

	int count;
	int texturesize;
	

public:
    CWater();
    virtual ~CWater();
	
    virtual HRESULT InitWorld( CDRGWrapper *pWrapper );
	virtual HRESULT DestroyWorld( void );

	virtual HRESULT UpdateWorld();
	virtual HRESULT RenderWorld( void );

	virtual HRESULT TargetChanged( void );

	void KeyDown(WPARAM wParam);
	void KeyUp(WPARAM wParam);

	void MouseLeftUp(WPARAM wParam, LPARAM lParam);
	void MouseLeftDown(WPARAM wParam, LPARAM lParam);
	void MouseRightUp(WPARAM wParam, LPARAM lParam);
	void MouseRightDown(WPARAM wParam, LPARAM lParam);
	void MouseMove(WPARAM wParam, LPARAM lParam);

private:
//	void SetupSampleBackgroundVerts();
	void SetupSampleCubeVerts(float scale);

//	LPDIRECT3DTEXTURE2 InitBumpMap( /*LPDIRECTDRAWSURFACE4 pddsBumpSrc*/ );
	void InitBumpMapArray(int Hres, int Vres, char *pArray);
	

};

#endif // !defined(AFX_DRGAPP_H__CFF24425_47D6_11D2_B1A5_00A0C90D2C1B__INCLUDED_)
